<?php
/* --------------------------------------------------------------
 ApiTestingServiceProvider.php 2019-07-16
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\ParameterFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\PropertyFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\SchemaFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\SimpleFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\RestDocRepository as RestDocRepositoryInterface;
use Gambio\Dragger\ApiTesting\SwaggerLike\Repository\EndpointMapper;
use Gambio\Dragger\ApiTesting\SwaggerLike\Repository\RestDocRepository;
use Gambio\Dragger\ApiTesting\SwaggerLike\Repository\SchemaMapper;
use Gambio\Dragger\ApiTesting\SwaggerLike\Resolver\ReferenceParser;
use Gambio\Dragger\ApiTesting\SwaggerLike\Resolver\ReferenceResolver;
use Gambio\Dragger\DatabaseFixture\FixtureImporter;
use Gambio\Dragger\DatabaseFixture\Processor\TableDataProcessor;
use League\Container\Container;
use League\Container\ServiceProvider\AbstractServiceProvider;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
use Symfony\Component\Yaml\Parser;
use Webmozart\Assert\Assert;

/**
 * Class ApiTestingServiceProvider
 *
 * @package Gambio\Dragger\ApiTesting\SwaggerLike
 * @property-read Container $container
 */
class ApiTestingServiceProvider extends AbstractServiceProvider
{
    /**
     * @var array
     */
    protected $provides = [
        RestDocRepositoryInterface::class
    ];
    
    /**
     * @var array
     */
    private $definitions;
    
    
    /**
     * ApiTestingServiceProvider constructor.
     *
     * @param string $definitionsPath
     */
    public function __construct(string $definitionsPath)
    {
        Assert::fileExists($definitionsPath);
        $iterator = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($definitionsPath, RecursiveDirectoryIterator::SKIP_DOTS),
            RecursiveIteratorIterator::CHILD_FIRST
        );
        $files    = [];
        foreach ($iterator as $file) {
            if ($file->isFile()) {
                $files[] = realpath($file->getPathname());
            }
        }
        $this->definitions = $files;
    }
    
    
    /**
     * Use the register method to register items with the container via the
     * protected $this->container property or the `getContainer` method
     * from the ContainerAwareTrait.
     *
     * @return void
     */
    public function register(): void
    {
        $this->container->add(RestDocRepositoryInterface::class, RestDocRepository::class)->addArguments(
            [
                Parser::class,
                EndpointMapper::class,
                SchemaMapper::class,
                ReferenceParser::class,
            ]
        );
        
        $this->registerMapperComponents();
        $this->registerRefComponents();
        $this->registerFactories();
        
        $this->leagueContainer->add(
            Connection::class,
            static function () {
                $dbHost = getenv('DB_HOST');
                $dbUser = getenv('DB_USER');
                $dbName = getenv('DB_NAME');
                $dbPass = getenv('DB_PASSWORD');
                $dbPass = $dbPass === '' ? '' : ':' . urlencode(($dbPass));
                $dsn    = "pdo-mysql://$dbUser$dbPass@$dbHost/$dbName?charset=UTF8";
                
                return DriverManager::getConnection(['url' => $dsn]);
            }
        );
        
        $this->leagueContainer->add(FixtureImporter::class)->addArguments(
            [Connection::class, TableDataProcessor::class]
        );
    }
    
    
    /**
     * Registers the mapper components.
     */
    private function registerMapperComponents(): void
    {
        $this->container->add(EndpointMapper::class)->addArguments([SimpleFactory::class, ParameterFactory::class]);
        $this->container->add(SchemaMapper::class)->addArguments(
            [
                SchemaFactory::class,
                PropertyFactory::class
            ]
        );
    }
    
    
    /**
     * Registers all components that somehow works with references.
     */
    private function registerRefComponents(): void
    {
        $this->container->add(Parser::class);
        $this->container->add(ReferenceResolver::class)->addArgument($this->definitions);
        $this->container->add(ReferenceParser::class)->addArguments([ReferenceResolver::class, Parser::class]);
    }
    
    
    /**
     * Registers the factory components.
     */
    private function registerFactories(): void
    {
        $this->container->add(SimpleFactory::class);
        $this->container->add(SchemaFactory::class);
        $this->container->add(ParameterFactory::class)->addArguments(
            [
                ReferenceParser::class,
                SchemaMapper::class,
            ]
        );
        $this->container->add(PropertyFactory::class)->addArgument(ReferenceParser::class);
    }
}