<?php
/* --------------------------------------------------------------
 SchemaDataFaker.php 2019-08-12
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Faker;

use Exception;
use Faker\Generator;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Properties;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Schema as SchemaInterface;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\TypedAndNamed;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\Schema;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaList;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaListScalar;
use Webmozart\Assert\Assert;
use function random_int;

/**
 * Class SchemaDataFaker
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Faker
 */
class SchemaDataFaker
{
    private const TYPE_GENERATOR_MAP = [
        'boolean' => 'fakeBoolean',
        'integer' => 'fakeInteger',
        'float'   => 'fakeFloat',
        'number'  => 'fakeFloat',
        'double'  => 'fakeFloat',
        'string'  => 'fakeString',
    ];
    
    /**
     * @var Generator
     */
    private $faker;
    
    /**
     * @var StringFaker
     */
    private $stringFaker;
    
    
    /**
     * SchemaDataFaker constructor.
     *
     * @param Generator $faker
     */
    public function __construct(Generator $faker)
    {
        $this->faker       = $faker;
        $this->stringFaker = new StringFaker($faker); // Todo: Fix and use DI.
    }
    
    
    /**
     * Creates random data set from given schema.
     *
     * @param SchemaInterface $schema
     * @param int             $nbDataSets
     *
     * @return array
     */
    public function fake(SchemaInterface $schema, int $nbDataSets = 1): array
    {
        Assert::greaterThanEq($nbDataSets, 1);
        
        $data = [];
        if ($schema instanceof Schema) {
            $data = $this->fakeProperties($schema->properties());
        }
        
        if ($schema instanceof SchemaListScalar) {
            for ($i = 0; $i < $nbDataSets; $i++) {
                $data[] = $this->createValue($schema);
            }
        }
        
        if ($schema instanceof SchemaList) {
            for ($i = 0; $i < $nbDataSets; $i++) {
                $data[] = $this->fake($schema->items());
            }
        }
        
        return $data;
    }
    
    
    /**
     * Uses given column names to fake a date time value.
     *
     * @param array $dateColumns
     */
    public function useDateColumns(array $dateColumns): void
    {
        $this->stringFaker->useDateColumns($dateColumns);
    }
    
    
    /**
     * Creates random data set from given properties.
     *
     * @param Properties $properties
     *
     * @return array
     */
    private function fakeProperties(Properties $properties): array
    {
        $data = [];
        foreach ($properties->items() as $property) {
            if ($property instanceof ListProperty) {
                $rngEntriesCount = $this->randomInt(2, 3);
                for ($i = 0; $i < $rngEntriesCount; $i++) {
                    $data[$property->name()][] = $this->createValue($property);
                }
            }
            
            if ($property instanceof ScalarProperty && !$property instanceof ListProperty) {
                $data[$property->name()] = $this->createValue($property);
            }
            
            if ($property instanceof ObjectListProperty) {
                $rngEntriesCount = $this->randomInt(2, 3);
                for ($i = 0; $i < $rngEntriesCount; $i++) {
                    $data[$property->name()][] = $this->fakeProperties($property->properties());
                }
            }
            
            if ($property instanceof ObjectProperty && !$property instanceof ObjectListProperty) {
                $data[$property->name()] = $this->fakeProperties($property->properties());
            }
        }
        
        return $data;
    }
    
    
    /**
     * Creates a random value in the expected type.
     *
     * @param TypedAndNamed $data
     *
     * @return mixed
     */
    private function createValue(TypedAndNamed $data)
    {
        $method = self::TYPE_GENERATOR_MAP[$data->type()];
        
        if ($method === 'fakeString') {
            return $this->fakeString($data->name());
        }
        
        return self::$method();
    }
    
    
    /**
     * Creates a bool random value by using faker generator.
     *
     * @return bool
     */
    private function fakeBoolean(): bool
    {
        return $this->faker->boolean;
    }
    
    
    /**
     * Creates a int random value by using faker generator.
     *
     * @return int
     */
    private function fakeInteger(): int
    {
        return $this->randomInt(1, 999);
    }
    
    
    /**
     * Creates a float random value by using faker generator.
     *
     * @return float
     */
    private function fakeFloat(): float
    {
        return $this->faker->randomFloat();
    }
    
    
    /**
     * Creates a string random value by using faker generator.
     *
     * @param string $name Used to guess type, eg. for dates.
     *
     * @return string
     */
    private function fakeString(string $name): string
    {
        return $this->stringFaker->fake($name);
    }
    
    
    /**
     * Creates a random integer value.
     *
     * @param int $min
     * @param int $max
     *
     * @return int
     */
    private function randomInt(int $min, int $max): int
    {
        try {
            return random_int($min, $max);
            // @codeCoverageIgnoreStart
        } catch (Exception $e) {
            /** @noinspection RandomApiMigrationInspection */
            return mt_rand($min, $max);
            // @codeCoverageIgnoreEnd
        }
    }
}