<?php
/* --------------------------------------------------------------
 ParameterFactory.php 2019-08-06
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Parameters;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Parameter;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Parameter\BodyParameter;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Parameter\ParameterCommon;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Parameter\PathParameter;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Reference\SchemaListReference;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Reference\SchemaReference;
use Gambio\Dragger\ApiTesting\SwaggerLike\Resolver\ReferenceParser;
use Gambio\Dragger\ApiTesting\SwaggerLike\Repository\SchemaMapper;
use InvalidArgumentException;
use function array_key_exists;

/**
 * Class ParameterFactory
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory
 */
class ParameterFactory
{
    private const KEY_ITEMS    = 'items';
    private const KEY_SCHEMA   = 'schema';
    private const KEY_REQUIRED = 'required';
    private const KEY_NAME     = 'name';
    private const KEY_TYPE     = 'type';
    private const KEY_PATH     = 'path';
    private const KEY_IN       = 'in';
    private const KEY_BODY     = 'body';
    
    /**
     * @var ReferenceParser
     */
    private $refParser;
    
    /**
     * @var SchemaMapper
     */
    private $schemaMapper;
    
    
    /**
     * ParameterFactory constructor.
     *
     * @param ReferenceParser $refParser
     * @param SchemaMapper    $schemaMapper
     */
    public function __construct(
        ReferenceParser $refParser,
        SchemaMapper $schemaMapper
    ) {
        $this->refParser    = $refParser;
        $this->schemaMapper = $schemaMapper;
    }
    
    
    /**
     * Creates parameter models based on the given data.
     *
     * @param array $data
     *
     * @return Parameter
     */
    public function create(array $data) : Parameter
    {
        $in     = $data[self::KEY_IN];
        $isList = array_key_exists(self::KEY_ITEMS, $data);
        $common = ParameterCommon::create($data[self::KEY_NAME], $in, $data[self::KEY_REQUIRED], $isList);
        
        if ($in === self::KEY_BODY) {
            $isSchema = array_key_exists(self::KEY_SCHEMA, $data);
            $isList   = ($isSchema && array_key_exists(self::KEY_ITEMS, $data[self::KEY_SCHEMA])) || $isList;
            
            // new common is needed, cause $isList may changed!
            $common = ParameterCommon::create($data[self::KEY_NAME], $in, $data[self::KEY_REQUIRED], $isList);
            
            $data = $isSchema ? $data[self::KEY_SCHEMA] : $data;
            
            $ref  = $isList ? SchemaListReference::fromArray($data) : SchemaReference::fromArray($data);
            $data = $this->refParser->parse($ref);
            
            $schema = $ref instanceof SchemaListReference ? $this->schemaMapper->mapList(
                $data
            ) : $this->schemaMapper->map($data);
            
            return BodyParameter::create($common, $schema);
        }
        
        if ($in === self::KEY_PATH) {
            $type = $isList ? $data[self::KEY_ITEMS][self::KEY_TYPE] : $data[self::KEY_TYPE];
            
            return PathParameter::create($common, $type);
        }
        
        throw new InvalidArgumentException('The "in" key of $data must be either "path" or "body".');
    }
    
    
    /**
     * Creates a parameters collection.
     *
     * @return Parameters
     */
    public function createParameters() : Parameters
    {
        return Parameters::create();
    }
}