<?php
/* --------------------------------------------------------------
 PropertyFactory.php 2019-08-06
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Properties;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Property;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\PropertyReference as PropertyReferenceInterface;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ReferencedListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ReferencedProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Reference\PropertyListReference;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Reference\PropertyReference;
use Gambio\Dragger\ApiTesting\SwaggerLike\Resolver\ReferenceParser;
use InvalidArgumentException;
use function array_key_exists;

/**
 * Class PropertyFactory
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory
 */
class PropertyFactory
{
    private const REQUIRED_KEY   = 'required';
    private const ITEMS_KEY      = 'items';
    private const PROPERTIES_KEY = 'properties';
    
    /**
     * @var ReferenceParser
     */
    private $refParser;
    
    
    /**
     * PropertyFactory constructor.
     *
     * @param ReferenceParser $refParser
     */
    public function __construct(ReferenceParser $refParser)
    {
        $this->refParser = $refParser;
    }
    
    
    /**
     * Creates a properties collection.
     *
     * @return Properties
     */
    public function createProperties() : Properties
    {
        return Properties::create();
    }
    
    
    /**
     * Creates a property model.
     * The model type is determined by the given data.
     *
     * @param string $name
     * @param array  $data
     *
     * @return Property
     */
    public function create(string $name, array $data) : Property
    {
        if (array_key_exists(self::ITEMS_KEY, $data)) {
            return $this->createListProperty($name, $data);
        }
        
        if (array_key_exists('$ref', $data)) {
            $reference = ReferencedProperty::create($name, $this->createPropertyReference($data));
            
            return $this->resolveEntity($reference->reference(), $name);
        }
        
        if (array_key_exists(self::PROPERTIES_KEY, $data)) {
            return $this->createNestedProperty($name, $data);
        }
        
        return ScalarProperty::create($name, $data['type'], array_key_exists('nullable', $data));
    }
    
    
    /**
     * @param string $name
     * @param array  $data
     *
     * @return ObjectListProperty|ObjectProperty|ScalarProperty
     */
    private function createListProperty(string $name, array $data) : Property
    {
        $items = $data[self::ITEMS_KEY];
        if (array_key_exists('$ref', $items)) {
            $reference = ReferencedListProperty::create($name, $this->createPropertyReference($data));
            
            return $this->resolveList($reference->reference(), $name);
        }
        if (array_key_exists('type', $items)) {
            if (array_key_exists('properties', $items)) {
                $properties = $this->createProperties();
                
                foreach ($items[self::PROPERTIES_KEY] as $propName => $propData) {
                    $property = $this->create($propName, $propData);
                    $properties->add($property);
                }
                $required = array_key_exists(self::REQUIRED_KEY, $items) ? $items[self::REQUIRED_KEY] : [];
                $nullable = array_key_exists('nullable', $items) && $items['nullable'];
                
                return ObjectListProperty::create($name, $properties, $required, $nullable);
            }
            
            return ListProperty::create($name, $items['type']);
        }
        
        throw new InvalidArgumentException('Invalid $data provided. Either "$ref", "type" key is needed.');
    }
    
    
    /**
     * Creates an object property model.
     *
     * @param string $name
     * @param array  $data
     *
     * @return ObjectProperty
     */
    private function createNestedProperty(string $name, array $data) : ObjectProperty
    {
        $properties = $this->createProperties();
        
        foreach ($data[self::PROPERTIES_KEY] as $propName => $propData) {
            $property = $this->create($propName, $propData);
            $properties->add($property);
        }
        
        $required = array_key_exists('required', $data) ? $data['required'] : [];
        $nullable = array_key_exists('nullable', $data) && $data['nullable'];
        $objProp  = ObjectProperty::create($name, $properties, $required, $nullable);
        $this->applyParentToProperties($objProp);
        
        return $objProp;
    }
    
    
    /**
     * Creates a property reference.
     *
     * @param array $data
     *
     * @return PropertyReferenceInterface
     */
    private function createPropertyReference(array $data) : PropertyReferenceInterface
    {
        if (array_key_exists(self::ITEMS_KEY, $data)) {
            return PropertyListReference::fromArray($data);
        }
        
        return PropertyReference::fromArray($data);
    }
    
    
    /**
     * Resolves reference and returns object list property from data.
     *
     * @param PropertyReferenceInterface $reference
     * @param string                     $name
     *
     * @return ObjectListProperty
     */
    private function resolveList(PropertyReferenceInterface $reference, string $name) : ObjectListProperty
    {
        /** @var ObjectListProperty $property */
        $property = ObjectListProperty::create(...$this->resolve($reference, $name));
        $this->applyParentToProperties($property);
        
        return $property;
    }
    
    
    /**
     * Resolves reference and returns object property from data.
     *
     * @param PropertyReferenceInterface $reference
     * @param string                     $name
     *
     * @return ObjectProperty
     */
    private function resolveEntity(PropertyReferenceInterface $reference, string $name) : ObjectProperty
    {
        $property = ObjectProperty::create(...$this->resolve($reference, $name));
        $this->applyParentToProperties($property);
        
        return $property;
    }
    
    
    /**
     * Resolves the given reference and returns data that is needed to create a new property.
     *
     * @param PropertyReferenceInterface $reference
     * @param string                     $name
     *
     * @return array
     */
    private function resolve(PropertyReferenceInterface $reference, string $name) : array
    {
        $referenceData = $this->refParser->parse($reference);
        
        $referenceProps = $this->createProperties();
        foreach ($referenceData[self::PROPERTIES_KEY] as $referenceName => $referenceDataSet) {
            $referenceProp = $this->create($referenceName, $referenceDataSet);
            $referenceProps->add($referenceProp);
        }
        $nullable = array_key_exists('nullable', $referenceData) && $referenceData['nullable'];
        $required = array_key_exists(self::REQUIRED_KEY, $referenceData) ? $referenceData[self::REQUIRED_KEY] : [];
        
        return [$name, $referenceProps, $required, $nullable];
    }
    
    
    /**
     * Applies given property as parent for all internal properties.
     *
     * @param ObjectProperty $property
     */
    private function applyParentToProperties(ObjectProperty $property) : void
    {
        foreach ($property->properties()->items() as $prop) {
            $prop->applyParent($property);
        }
    }
}