<?php
/* --------------------------------------------------------------
 SchemaMapper.php 2019-07-04
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Repository;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Schema;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\SchemaList;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\PropertyFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\SchemaFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\Schema as MappedSchema;
use Webmozart\Assert\Assert;
use function array_key_exists;

/**
 * Class SchemaMapper
 *
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Repository
 */
class SchemaMapper
{
    private const NON_SCALAR_TYPES = ['array', 'object'];
    
    /**
     * @var SchemaFactory
     */
    private $schemaFactory;
    
    /**
     * @var PropertyFactory
     */
    private $propertyFactory;
    
    
    /**
     * SchemaMapper constructor.
     *
     * @param SchemaFactory   $schemaFactory
     * @param PropertyFactory $propertyFactory
     */
    public function __construct(SchemaFactory $schemaFactory, PropertyFactory $propertyFactory)
    {
        $this->schemaFactory   = $schemaFactory;
        $this->propertyFactory = $propertyFactory;
    }
    
    
    /**
     * Maps data to Schema model.
     *
     * @param array $data
     *
     * @return Schema
     */
    public function map(array $data) : Schema
    {
        Assert::notEmpty($data);
        
        if (array_key_exists('items', $data)) {
            $items = $this->mapToSchema($data['items']);
            
            return $this->schemaFactory->createSchemaList($data['title'], $items);
        }
        
        return $this->mapToSchema($data);
    }
    
    
    /**
     * Maps data to schema list.
     *
     * @param array $data
     *
     * @return SchemaList
     */
    public function mapList(array $data) : SchemaList
    {
        $title = $data['title'];
        if (array_key_exists('type', $data) && !in_array($data['type'], self::NON_SCALAR_TYPES, true)) {
            return $this->schemaFactory->createSchemaListScalar($title, $data['type']);
        }
        
        $listTitle = $title . 'List';
        $items     = $this->mapToSchema($data);
        
        return $this->schemaFactory->createSchemaList($listTitle, $items);
    }
    
    
    /**
     * Maps given data to schema.
     *
     * @param array $dataSet
     *
     * @return MappedSchema
     */
    private function mapToSchema(
        array $dataSet
    ) : MappedSchema {
        $properties = $this->propertyFactory->createProperties();
        $title      = $dataSet['title'];
        $required   = $dataSet['required'] ?? null;
        
        foreach ($dataSet['properties'] as $name => $prop) {
            $property = $this->propertyFactory->create($name, $prop);
            $properties->add($property);
        }
        
        return $this->schemaFactory->createSchema($title, $properties, $required);
    }
}