<?php
/* --------------------------------------------------------------
 AddedDataSetValidator.php 2019-08-13
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DBALException;
use Gambio\Dragger\ApiTesting\SwaggerLike\ApiClient;
use Gambio\Dragger\ApiTesting\SwaggerLike\Exception\RequestFailedException;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\SimpleFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\SimpleEventListener;
use Gambio\Dragger\ApiTesting\SwaggerLike\TestCase\AbstractApiTestCase;
use Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Post\PostAffectedTables;
use Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Post\PostTestVerification;

/**
 * Class AddedDataSetValidator
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Validator
 */
class AddedDataSetValidator
{
    private const COUNT_QUERY = 'SELECT COUNT(*) as `count` FROM `%s`;';
    
    /**
     * @var Failures
     */
    private $failures;
    
    /**
     * @var Connection
     */
    private $connection;
    
    /**
     * @var ApiClient
     */
    private $client;
    
    
    /**
     * AddedDataSetValidator constructor.
     *
     * @param Failures   $failures
     * @param Connection $connection
     * @param ApiClient  $client
     */
    public function __construct(Failures $failures, Connection $connection, ApiClient $client)
    {
        $this->failures   = $failures;
        $this->connection = $connection;
        $this->client     = $client;
    }
    
    
    /**
     * Validates (and also actually performing) the post request.
     *
     * @param PostTestVerification $verification
     *
     * @return bool
     */
    public function validate(PostTestVerification $verification): bool
    {
        $initialCount = $this->countDataSets($verification->affectedTables());
        
        try {
            $response = $this->client->requestOrFail(
                $verification->requestMethod(),
                $verification->requestPath(),
                $verification->requestOptions()
            );
            SimpleEventListener::trigger(AbstractApiTestCase::EVENT_RECEIVED_RESPONSE, $response);
            SimpleEventListener::remove(AbstractApiTestCase::EVENT_RECEIVED_RESPONSE);
        } catch (RequestFailedException $e) {
            $this->failures->add(SimpleFailure::create($e->getMessage()));
        }
        
        $actualCount = $this->countDataSets($verification->affectedTables());
        $this->verifyCount($verification->affectedTables(), $initialCount, $actualCount, $verification->nbDataSets());
        
        return !$this->failures->failed();
    }
    
    
    /**
     * Verifies the final row count.
     * This is done by summing the product of
     * $nbDataSets and $multiplier of table with $initialCount of table.
     *
     * @param PostAffectedTables $affectedTables
     * @param array              $initialCount
     * @param array              $actualCount
     * @param int                $nbDataSets
     */
    private function verifyCount(
        PostAffectedTables $affectedTables,
        array $initialCount,
        array $actualCount,
        int $nbDataSets
    ): void {
        foreach ($affectedTables->items() as $table => $multiplier) {
            $expected = $initialCount[$table] + $nbDataSets * $multiplier;
            $actual   = $actualCount[$table];
            
            if ($expected !== $actual) {
                $msg = <<<MSG
Failed asserting that row count for table "{$table}" ({$actual}) matches expected row count ({$expected}).
MSG;
                $this->failures->add(SimpleFailure::create($msg));
            }
        }
    }
    
    
    /**
     * Calculates row counts.
     *
     * @param PostAffectedTables $affectedTables
     *
     * @return array
     */
    private function countDataSets(PostAffectedTables $affectedTables): array
    {
        $rowCount = [];
        
        foreach ($affectedTables->tablesOnly() as $table) {
            try {
                $countData = $this->connection->query(sprintf(self::COUNT_QUERY, $table))->fetch();
                $count     = (int)$countData['count'];
                
                $rowCount[$table] = $count;
            } catch (DBALException $e) {
            }
        }
        
        return $rowCount;
    }
    
    
    /**
     * @return string
     */
    public function failures(): string
    {
        return $this->failures->toString();
    }
}