<?php
/* --------------------------------------------------------------
 ChangedDataSetValidator.php 2019-08-20
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DBALException;
use Gambio\Dragger\ApiTesting\SwaggerLike\ApiClient;
use Gambio\Dragger\ApiTesting\SwaggerLike\Exception\RequestFailedException;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\SimpleFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\SimpleEventListener;
use Gambio\Dragger\ApiTesting\SwaggerLike\TestCase\AbstractApiTestCase;
use Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Put\PutAffectedIds;
use Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Put\PutAffectedTables;
use Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Put\PutTestVerificationBak;

/**
 * Class ChangedDataSetValidator
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Validator
 */
class ChangedDataSetValidator
{
    private const COUNT_QUERY  = 'SELECT COUNT(*) AS `count` FROM `%s`;';
    private const SELECT_QUERY = 'SELECT * FROM `%s` WHERE `%s` = "%s";';
    
    /**
     * @var Failures
     */
    private $failures;
    
    /**
     * @var Connection
     */
    private $connection;
    
    /**
     * @var ApiClient
     */
    private $apiClient;
    
    
    /**
     * ChangedDataSetValidator constructor.
     *
     * @param Failures   $failures
     * @param Connection $connection
     * @param ApiClient  $apiClient
     */
    public function __construct(Failures $failures, Connection $connection, ApiClient $apiClient)
    {
        $this->failures   = $failures;
        $this->connection = $connection;
        $this->apiClient  = $apiClient;
    }
    
    
    /**
     * @return string
     */
    public function failures(): string
    {
        return $this->failures->toString();
    }
    
    
    /**
     * Validates put request.
     * Checks if expected data was changed.
     *
     * @param PutTestVerificationBak $verification
     *
     * @return bool
     */
    public function validate(PutTestVerificationBak $verification): bool
    {
        // setup test first
        // the setup is has two phases
        // first of all, all rows of affected tables will be counted
        // then, a general query will fetch data from those tables and stores their result
        $initialCount = $this->countDataSets($verification->affectedTables());
        $initialData  = $this->tablesData($verification->affectedTables(), $verification->affectedIds());
        
        // execute request to be tested
        // simple execution of request to be tested
        try {
            $actualResponse = $this->apiClient->requestOrFail(
                $verification->requestMethod(),
                $verification->requestPath(),
                $verification->requestOptions()
            );
            SimpleEventListener::trigger(AbstractApiTestCase::EVENT_RECEIVED_RESPONSE, $actualResponse);
            SimpleEventListener::remove(AbstractApiTestCase::EVENT_RECEIVED_RESPONSE);
        } catch (RequestFailedException $e) {
            $this->failures->add(SimpleFailure::create($e->getMessage()));
        }
        
        // verify results
        // again, the verification has two phases (like setup)
        // first of all, we verify that no new data sets was added
        // then, we execute the same query like in setup and verify that the result changed
        $actualCount = $this->countDataSets($verification->affectedTables());
        $actualData  = $this->tablesData($verification->affectedTables(), $verification->affectedIds());
        
        if ($initialCount !== $actualCount) {
            $this->failures->add(SimpleFailure::create('Foo.'));
        }
        if ($initialData === $actualData) {
            $this->failures->add(SimpleFailure::create('Bar.'));
        }
        
        return !$this->failures->failed();
    }
    
    
    /**
     * Calculates row counts.
     *
     * @param PutAffectedTables $affectedTables
     *
     * @return array
     */
    private function countDataSets(PutAffectedTables $affectedTables): array
    {
        $rowCount = [];
        
        foreach ($affectedTables->tablesOnly() as $table) {
            try {
                $countData = $this->connection->query(sprintf(self::COUNT_QUERY, $table))->fetch();
                $count     = (int)$countData['count'];
                
                $rowCount[$table] = $count;
            } catch (DBALException $e) {
            }
        }
        
        return $rowCount;
    }
    
    
    /**
     * Pretty general sql query, filtered by identifier values from verifications.
     * Its used to first fetch initial data and then fetch (hopefully) updated data.
     *
     * @param PutAffectedTables $affectedTables
     * @param PutAffectedIds    $affectedIds
     *
     * @return array
     */
    public function tablesData(PutAffectedTables $affectedTables, PutAffectedIds $affectedIds): array
    {
        $data = [];
        
        foreach ($affectedTables->items() as $table => $identifier) {
            
            foreach ($affectedIds->ids() as $verificationId) {
                try {
                    $data[$table][$verificationId] = $this->connection->query(
                        sprintf(self::SELECT_QUERY, $table, $identifier, $verificationId)
                    )->fetchAll();
                } catch (DBALException $e) {
                }
            }
        }
        
        return $data;
    }
}