<?php
/* --------------------------------------------------------------
 AbstractDeleteStrategy.php 2019-08-30
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Delete;

use Doctrine\DBAL\Connection;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\SimpleFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Delete\Expect;

/**
 * Class AbstractDeleteStrategy
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Delete
 */
class AbstractDeleteStrategy
{
    /**
     * @var Connection
     */
    protected $connection;
    
    /**
     * @var Failures
     */
    protected $failures;
    
    
    /**
     * CombinedIdStrategy constructor.
     *
     * @param Connection $connection
     */
    private function __construct(Connection $connection)
    {
        $this->connection = $connection;
        $this->failures   = Failures::create();
    }
    
    
    /**
     * Factory method for CombinedIdStrategy.
     *
     * @param Connection $connection
     *
     * @return AbstractDeleteStrategy
     */
    public static function create(Connection $connection): self
    {
        return new static($connection);
    }
    
    
    /**
     * @return string
     */
    public function failures(): string
    {
        return $this->failures->toString();
    }
    
    
    /**
     * Checks row counts if they match $expectedCount.
     * Adds failure if expectation fails.
     *
     * @param bool     $exists
     * @param string   $additionalErrMsg
     * @param Expect[] $expectations
     */
    protected function checkDataSetsExist(
        bool $exists,
        string $additionalErrMsg,
        Expect ...$expectations
    ): void {
        foreach ($expectations as $expectation) {
            $qb = $this->connection->createQueryBuilder();
            
            $table       = $expectation->table();
            $identifiers = [];
            $ids         = [];
            
            // builds query
            
            $query = $qb->select('*')->from($table);
            foreach ($expectation->columns() as $identifier => $id) {
                $identifiers[] = $identifier;
                $ids[]         = $id;
                
                $where = "{$this->connection->quoteIdentifier($identifier)} = {$qb->createNamedParameter($id)}";
                $query->where($where);
            }
            
            $result    = $query->execute()->fetchAll();
            $foundData = !empty($result);
            if ($foundData === !$exists) { // verification
                // create err msg
                $identifiersStr = implode(', ', $identifiers);
                $idsStr         = implode(', ', $ids);
                
                $hl = 'Failed asserting that data set ';
                $hl .= $exists ? 'exists' : 'not exists' . '.';
                
                $message = <<<MSG
$hl
Table:       $table
Identifiers: $identifiersStr
Id:          $idsStr
MSG;
                if ($additionalErrMsg !== '') {
                    $message .= PHP_EOL . $additionalErrMsg;
                }
                $this->failures->add(SimpleFailure::create($message));
            }
        }
    }
}