<?php
/* --------------------------------------------------------------
 PropertiesValidator.php 2019-08-16
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema;

use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\PropertyFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\SchemaFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Properties;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Property;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use function array_filter;
use function array_keys;
use function count;
use function in_array;
use function is_array;
use function is_string;
use function range;

/**
 * Class PropertiesValidator
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema
 */
class PropertiesValidator
{
    private const EXCLUDED_PROPERTIES = [
        '_links'
    ];
    
    /**
     * @var ScalarPropertyValidator
     */
    private $scalarValidator;
    
    /**
     * @var RequiredValidator
     */
    private $requiredValidator;
    
    /**
     * @var Failures
     */
    private $failures;
    
    
    /**
     * PropertiesValidator constructor.
     *
     * @param ScalarPropertyValidator $scalarValidator
     * @param RequiredValidator       $requiredValidator
     * @param Failures                $failures
     */
    public function __construct(
        ScalarPropertyValidator $scalarValidator,
        RequiredValidator $requiredValidator,
        Failures $failures
    ) {
        $this->scalarValidator   = $scalarValidator;
        $this->requiredValidator = $requiredValidator;
        $this->failures          = $failures;
    }
    
    
    /**
     * Validates all given properties.
     *
     * @param Properties    $properties
     * @param array         $responseItem
     * @param Property|null $parent
     *
     * @return bool
     */
    public function validate(Properties $properties, array $responseItem, Property $parent = null) : bool
    {
        $failed = false;
        
        foreach ($responseItem as $name => $value) {
            if ($properties->has($name)) {
                $property = $properties->get($name);
                if (!$this->validateProperty($property, $value)) {
                    $failed = true;
                }
            } elseif (!in_array($name, self::EXCLUDED_PROPERTIES, true)) {
                $this->failures->add(SchemaFailure::missingInSchema($name, $parent));
            }
        }
        
        return !$failed;
    }
    
    
    /**
     * Validates the given property schema and $value if they match.
     *
     * @param Property $property
     * @param mixed    $value
     *
     * @return bool
     */
    private function validateProperty(Property $property, $value) : bool
    {
        $failed = false;
        
        if (null === $value && $property->isNullable()) {
            return true;
        }
        
        if ($property instanceof ObjectListProperty) {
            if (!$this->isList($value)) {
                $this->failures->add(PropertyFailure::notList($property, $this->isMap($value)));
                
                return false;
            }
            
            foreach ($value as $val) {
                if (!$this->requiredValidator->validate($property, $val)) {
                    $failed = true;
                }
                if (!$this->validate($property->properties(), $val, $property)) {
                    $failed = true;
                }
            }
        }
        if (!$property instanceof ObjectListProperty && $property instanceof ObjectProperty) {
            if (!$this->isMap($value)) {
                $this->failures->add(PropertyFailure::notMap($property, $this->isList($value)));
                
                return false;
            }
            
            $this->requiredValidator->validate($property, $value);
            $this->validate($property->properties(), $value, $property);
        }
        
        if (($property instanceof ScalarProperty) && !$this->scalarValidator->validate($property, $value)) {
            $failed = true;
        }
        
        return !$failed;
    }
    
    
    /**
     * Checks if $array is an indexed array.
     *
     * @param mixed $array
     *
     * @return bool
     */
    private function isList($array) : bool
    {
        // allow empty array
        if (is_array($array) && count($array) === 0) {
            return true;
        }
        
        return !(!is_array($array) || !$array || array_keys($array) !== range(0, count($array) - 1));
    }
    
    
    /**
     * Checks if $array is an associative array.
     *
     * @param mixed $array
     *
     * @return bool
     */
    private function isMap($array) : bool
    {
        // allow empty array
        if (is_array($array) && count($array) === 0) {
            return true;
        }
        
        return !(!is_array($array)
                 || !$array
                 || array_keys($array) !== array_filter(
                array_keys($array), static function ($key) {
                return is_string($key);
            }
            ));
    }
}