<?php
/* --------------------------------------------------------------
 ScalarPropertyValidator.php 2019-08-15
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema;

use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\PropertyFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use function array_filter;
use function array_keys;
use function count;
use function gettype;
use function in_array;
use function is_array;
use function is_string;
use function range;

/**
 * Class ScalarPropertyValidator
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema
 */
class ScalarPropertyValidator
{
    private const ALLOWED_NUM_TYPES = [
        'double',
        'integer'
    ];
    
    /**
     * @var Failures
     */
    private $failures;
    
    
    /**
     * ScalarPropertyValidator constructor.
     *
     * @param Failures $failures
     */
    public function __construct(Failures $failures)
    {
        $this->failures = $failures;
    }
    
    
    public function validate(ScalarProperty $property, $value) : bool
    {
        $initialFailures = $this->failures->count();
        
        if (null === $value && $property->isNullable()) {
            return true;
        }
        
        if ($property instanceof ListProperty) {
            // allow empty array
            if (is_array($value) && count($value) === 0) {
                return true;
            }
            
            if (!$this->isList($value)) {
                $this->failures->add(PropertyFailure::notList($property, $this->isMap($value)));
                
                return false;
            }
            
            foreach ($value as $val) {
                $this->validateProperty($property, $val);
            }
            
            return $initialFailures === $this->failures->count();
        }
        $this->validateProperty($property, $value);
        
        return $initialFailures === $this->failures->count();
    }
    
    
    private function validateProperty(ScalarProperty $property, $value) : void
    {
        $type = gettype($value);
        
        if ($property->type() === 'number' && in_array($type, self::ALLOWED_NUM_TYPES, true)) {
            return;
        }
        
        if ($property->type() !== gettype($value)) {
            $this->failures->add(PropertyFailure::wrongType($property, $value));
        }
    }
    
    
    /**
     * Checks if $array is an indexed array.
     *
     * @param mixed $array
     *
     * @return bool
     */
    private function isList($array) : bool
    {
        return !(!is_array($array) || !$array || array_keys($array) !== range(0, count($array) - 1));
    }
    
    
    /**
     * Checks if $array is an associative array.
     *
     * @param mixed $array
     *
     * @return bool
     */
    private function isMap($array) : bool
    {
        // @codeCoverageIgnoreStart
        return !(!is_array($array)
                 || !$array
                 || array_keys($array) !== array_filter(
                array_keys($array), static function ($key) {
                return is_string($key);
            }
            ));
        // @codeCoverageIgnoreEnd
    }
}