<?php
/* --------------------------------------------------------------
 SchemaValidator.php 2019-08-16
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema;

use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\SchemaFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Schema as SchemaInterface;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\Schema;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaList;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaListScalar;
use function array_filter;
use function array_keys;
use function count;
use function in_array;
use function is_array;
use function is_string;
use function range;

/**
 * Class SchemaValidator
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema
 */
class SchemaValidator
{
    private const ALLOWED_NUM_TYPES = [
        'double',
        'integer'
    ];
    
    /**
     * @var PropertiesValidator
     */
    private $propertiesValidator;
    
    /**
     * @var RequiredValidator
     */
    private $requiredValidator;
    
    /**
     * @var Failures
     */
    private $failures;
    
    
    /**
     * SchemaValidator constructor.
     *
     * @param PropertiesValidator $propertiesValidator
     * @param RequiredValidator   $requiredValidator
     * @param Failures            $failures
     */
    public function __construct(
        PropertiesValidator $propertiesValidator,
        RequiredValidator $requiredValidator,
        Failures $failures
    ) {
        $this->propertiesValidator = $propertiesValidator;
        $this->requiredValidator   = $requiredValidator;
        $this->failures            = $failures;
    }
    
    
    /**
     * Validates the given schema and response.
     *
     * @param SchemaInterface $schema
     * @param array           $response
     *
     * @return bool
     */
    public function validate(SchemaInterface $schema, array $response) : bool
    {
        $this->failures->reset();
        $failed = false;
        
        if (!$this->validateSchema($schema, $response)) {
            $failed = true;
        }
        if (!$this->validateSchemaList($schema, $response)) {
            $failed = true;
        }
        if (!$this->validateSchemaListScalar($schema, $response)) {
            $failed = true;
        }
        
        return !$failed;
    }
    
    
    /**
     * @return string
     */
    public function failures() : string
    {
        return $this->failures->toString();
    }
    
    
    /**
     * Validates schema.
     *
     * @param SchemaInterface $schema
     * @param array           $response
     *
     * @return bool
     */
    private function validateSchema(SchemaInterface $schema, array $response) : bool
    {
        $failed = false;
        if ($schema instanceof Schema) {
            if (!$this->isMap($response)) {
                $this->failures->add(SchemaFailure::notMap($this->isList($response)));
                
                return false;
            }
            
            if (!$this->requiredValidator->validate($schema, $response)) {
                $failed = true;
            }
            if (!$this->propertiesValidator->validate($schema->properties(), $response)) {
                $failed = true;
            }
        }
        
        return !$failed;
    }
    
    
    /**
     * Validates schema lists.
     *
     * @param SchemaInterface $schema
     * @param array           $response
     *
     * @return bool
     */
    private function validateSchemaList(SchemaInterface $schema, array $response) : bool
    {
        $failed = false;
        if ($schema instanceof SchemaList) {
            if (!$this->isList($response)) {
                $this->failures->add(SchemaFailure::notList($this->isMap($response)));
                
                return false;
            }
            
            foreach ($response as $item) {
                if (!$this->requiredValidator->validate($schema->items(), $item)) {
                    $failed = true;
                }
                if (!$this->propertiesValidator->validate($schema->items()->properties(), $item)) {
                    $failed = true;
                }
            }
        }
        
        return !$failed;
    }
    
    
    /**
     * Validates scalar schema lists.
     *
     * @param SchemaInterface $schema
     * @param array           $response
     *
     * @return bool
     */
    private function validateSchemaListScalar(SchemaInterface $schema, array $response) : bool
    {
        $failed = false;
        
        if ($schema instanceof SchemaListScalar) {
            if (!$this->isList($response)) {
                $this->failures->add(SchemaFailure::notList($this->isMap($response)));
                
                return false;
            }
            
            foreach ($response as $item) {
                $type = gettype($item);
                
                if ($schema->type() === 'number' && in_array($type, self::ALLOWED_NUM_TYPES, true)) {
                    continue;
                }
                
                if ($type !== $schema->type()) {
                    $this->failures->add(SchemaFailure::wrongType($schema, $item));
                    $failed = true;
                }
            }
        }
        
        return !$failed;
    }
    
    
    /**
     * Checks if $array is an indexed array.
     *
     * @param mixed $array
     *
     * @return bool
     */
    private function isList($array) : bool
    {
        return !(!is_array($array) || !$array || array_keys($array) !== range(0, count($array) - 1));
    }
    
    
    /**
     * Checks if $array is an associative array.
     *
     * @param mixed $array
     *
     * @return bool
     */
    private function isMap($array) : bool
    {
        return !(!is_array($array)
                 || !$array
                 || array_keys($array) !== array_filter(
                array_keys($array), static function ($key) {
                return is_string($key);
            }
            ));
    }
}