<?php
/* --------------------------------------------------------------
 PostVerifications.php 2019-08-13
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Verification\Post;

use Faker\Factory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Faker\SchemaDataFaker;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Endpoint;
use Gambio\Dragger\ApiTesting\SwaggerLike\Traits\EndpointUrlResolver;

/**
 * Class PostVerifications
 * @package Gambio\Dragger\ApiTesting\SwaggerLike\Verification
 */
class PostTestVerification
{
    use EndpointUrlResolver;
    
    /**
     * @var Endpoint
     */
    private $endpoint;
    
    /**
     * @var PostAffectedTables
     */
    private $affectedTables;
    
    /**
     * @var int
     */
    private $nbDataSets;
    
    /**
     * @var SchemaDataFaker
     */
    private $faker;
    
    /**
     * @var array|null
     */
    private $requestBody;
    
    /**
     * @var array
     */
    private $urlPlaceholders = [];
    
    
    /**
     * PostTestVerification constructor.
     *
     * @param Endpoint           $endpoint
     * @param PostAffectedTables $affectedTables
     * @param int                $nbDataSets
     * @param SchemaDataFaker    $faker
     */
    private function __construct(
        Endpoint $endpoint,
        PostAffectedTables $affectedTables,
        int $nbDataSets,
        SchemaDataFaker $faker
    ) {
        $this->endpoint       = $endpoint;
        $this->affectedTables = $affectedTables;
        $this->nbDataSets     = $nbDataSets;
        $this->faker          = $faker;
    }
    
    
    /**
     * Factory method for PostTestVerification.
     *
     * @param Endpoint           $endpoint
     * @param PostAffectedTables $affectedTables
     * @param int                $nbDataSets
     *
     * @return PostTestVerification
     */
    public static function create(Endpoint $endpoint, PostAffectedTables $affectedTables, int $nbDataSets = 1): self
    {
        $faker = new SchemaDataFaker(Factory::create('de_DE'));
        
        return new static($endpoint, $affectedTables, $nbDataSets, $faker);
    }
    
    
    /**
     * Uses given column names to fake a date time value.
     *
     * @param array $dateColumns
     */
    public function useDateColumns(array $dateColumns): void
    {
        $this->faker->useDateColumns($dateColumns);
    }
    
    
    /**
     * @return Endpoint
     */
    public function endpoint(): Endpoint
    {
        return $this->endpoint;
    }
    
    
    /**
     * @return string
     */
    public function requestMethod(): string
    {
        return $this->endpoint->method();
    }
    
    
    /**
     * @return string
     */
    public function requestPath(): string
    {
        return $this->resolveEndpointUrl($this->endpoint, $this->urlPlaceholders);
    }
    
    
    /**
     * @return array
     */
    public function requestOptions(): array
    {
        $body = $this->endpoint->bodyParameter();
        
        if ($body) {
            $data = $this->requestBody ? : $this->faker->fake($body->schema(), $this->nbDataSets);
            
            return ['json' => $data];
        }
        
        return [];
    }
    
    
    /**
     * @return PostAffectedTables
     */
    public function affectedTables(): PostAffectedTables
    {
        return $this->affectedTables;
    }
    
    
    /**
     * @return int
     */
    public function nbDataSets(): int
    {
        return $this->nbDataSets;
    }
    
    
    /**
     * Provides interface for url placeholders.
     * Can be removed by ::removeUrlPlaceholders.
     *
     * @param array|null $urlPlaceholders
     *
     * @see removeUrlPlaceholders
     */
    public function withUrlPlaceholders(array $urlPlaceholders): void
    {
        $this->urlPlaceholders = $urlPlaceholders;
    }
    
    
    /**
     * Reset custom url placeholder.
     */
    public function removeUrlPlaceholders(): void
    {
        $this->withUrlPlaceholders([]);
    }
    
    
    /**
     * Provides interfaces to for own request body data.
     * Can be removed by ::removeCustomRequestBody.
     *
     * @param array|null $requestBody
     *
     * @see removeCustomRequestBody
     */
    public function withCustomRequestBody(?array $requestBody): void
    {
        $this->requestBody = $requestBody;
    }
    
    
    /**
     * Removes the custom request body and uses faked data from body param in schema.
     */
    public function removeCustomRequestBody(): void
    {
        $this->withCustomRequestBody(null);
    }
}