<?php
/* --------------------------------------------------------------
 DeletedFilesComparator.php 2020-04-21
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

namespace Gambio\Dragger\PackageComparator;

use Gambio\Dragger\PackageComparator\Factories\PackageFactory;
use Gambio\Dragger\PackageComparator\Model\ComparePackage;
use Gambio\Dragger\PackageComparator\Processors\DiffProcessor;
use Gambio\Dragger\PackageComparator\Processors\DiffVerifyProcessor;
use Gambio\Dragger\PackageComparator\Repositories\PackageRepository;
use Gambio\Dragger\PackageComparator\Utility\DirectoryReader;

/**
 * Class DeletedFilesComparator
 * @package Gambio\Dragger\PackageComparator
 *
 * @codeCoverageIgnore
 */
class DeletedFilesComparator
{
    /**
     * @var PackageRepository
     */
    private $packageRepository;
    
    /**
     * @var DiffProcessor
     */
    private $diffProcessor;
    
    /**
     * @var DiffVerifyProcessor
     */
    private $diffVerifyProcessor;
    
    /**
     * @var string
     */
    private $sourcePackage;
    
    /**
     * @var array
     */
    private $comparePackages;
    
    /**
     * @var array
     */
    private $comparisonResult;
    
    /**
     * @var bool
     */
    private $verbose = false;
    
    
    /**
     * PackageComparator constructor.
     *
     * @param PackageRepository   $packageRepository
     * @param DiffProcessor       $diffProcessor
     * @param DiffVerifyProcessor $diffVerifyProcessor
     * @param string              $sourcePackage
     * @param array               $comparePackages
     */
    private function __construct(
        PackageRepository $packageRepository,
        DiffProcessor $diffProcessor,
        DiffVerifyProcessor $diffVerifyProcessor,
        string $sourcePackage,
        array $comparePackages
    ) {
        $this->packageRepository   = $packageRepository;
        $this->diffProcessor       = $diffProcessor;
        $this->diffVerifyProcessor = $diffVerifyProcessor;
        $this->sourcePackage       = $sourcePackage;
        $this->comparePackages     = $comparePackages;
    }
    
    
    /**
     * Factory method for PackageComparator.
     *
     * @param string $sourcePackage
     * @param array  $comparePackages
     *
     * @return static
     */
    public static function create(string $sourcePackage, array $comparePackages): self
    {
        $packageFactory      = new PackageFactory();
        $packageRepository   = new PackageRepository($packageFactory);
        $directoryReader     = new DirectoryReader();
        $diffProcessor       = new DiffProcessor($directoryReader);
        $diffVerifyProcessor = new DiffVerifyProcessor($directoryReader);
        
        return new static($packageRepository, $diffProcessor, $diffVerifyProcessor, $sourcePackage, $comparePackages);
    }
    
    
    /**
     * Compares the source package with all compare packages.
     *
     * The list of differences is internally stored and will be returned afterwards.
     * There are additional methods to work with the comparison result.
     *
     * @param array $excludes
     *
     * @return array
     * @throws Exceptions\PathNotFoundException
     */
    public function compare(array $excludes = []): array
    {
        $sourcePackage   = $this->createComparePackage('source', $this->sourcePackage, $excludes);
        $comparePackages = [];
        foreach ($this->comparePackages as $index => $comparePackagePath) {
            $number = $index + 1;
            
            $comparePackages[] = $this->createComparePackage(
                "compare-package-{$number}",
                $comparePackagePath,
                $excludes
            );
        }
        
        $differences = [];
        foreach ($comparePackages as $comparePackage) {
            $this->log("compare ({$sourcePackage->name()}) with ({$comparePackage->name()})");
            $differences[] = $this->diffProcessor->process($sourcePackage, $comparePackage);
        }
        $differences = \array_merge(...$differences);
        $differences = \array_unique($differences);
        sort($differences);
        
        $this->log('verify comparison result');
        $this->diffVerifyProcessor->process($sourcePackage, $differences);
        
        $this->comparisonResult = $differences;
        
        return $this->comparisonResult;
    }
    
    
    /**
     * Changes the compare packages.
     *
     * @param string $sourcePackage
     * @param array  $comparePackages
     */
    public function changeComparePackages(string $sourcePackage, array $comparePackages): void
    {
        $this->log('change compare packages');
        
        $this->sourcePackage   = $sourcePackage;
        $this->comparePackages = $comparePackages;
    }
    
    
    /**
     * Manipulates the comparison result.
     *
     * @param callable $callback
     *
     * @return array
     */
    public function manipulateComparison(callable $callback): array
    {
        $this->log('manipulate comparison');
        $this->comparisonResult = array_map($callback, $this->comparisonResult);
        
        return $this->comparisonResult;
    }
    
    
    /**
     * Returns the comparison result.
     *
     * @return array
     */
    public function getComparison(): array
    {
        return $this->comparisonResult;
    }
    
    
    /**
     * Saves the comparison result at the given file path.
     * Returns false in case of a failure.
     *
     * @param string $savePath
     *
     * @return bool
     */
    public function saveComparison(string $savePath): bool
    {
        $this->log("save comparison data to ({$savePath})");
        
        return @file_put_contents($savePath, implode("\n", $this->comparisonResult)) !== false;
    }
    
    
    /**
     * Enables verbosity.
     */
    public function verbose(): void
    {
        $this->verbose = true;
    }
    
    
    /**
     * Disables verbosity.
     */
    public function silent(): void
    {
        $this->verbose = false;
    }
    
    
    /**
     * Creates a compare package and additionally logs, if verbosity is enabled.
     *
     * @param string $packageName
     * @param string $path
     * @param array  $excludes
     *
     * @return ComparePackage
     * @throws Exceptions\PathNotFoundException
     */
    private function createComparePackage(string $packageName, string $path, array $excludes): ComparePackage
    {
        $message = "create compare package ({$packageName})";
        $this->log($message);
        
        return $this->packageRepository->getComparePackage($packageName, $path, $excludes);
    }
    
    
    /**
     * @param string $message
     */
    private function log(string $message): void
    {
        if ($this->verbose) {
            $time = date('H:i:s');
            echo "[\033[0;37m{$time}\033[0m] - {$message}\n";
        }
    }
}