<?php
/* --------------------------------------------------------------
 DiffProcessor.php 2020-04-27
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

namespace Gambio\Dragger\PackageComparator\Processors;

use Gambio\Dragger\PackageComparator\Model\ComparePackage;
use Gambio\Dragger\PackageComparator\Model\Directory;
use Gambio\Dragger\PackageComparator\Utility\DirectoryReader;
use function array_merge;
use function array_unique;

/**
 * Class DiffProcessor
 * @package Gambio\Dragger\PackageComparator\Processors
 *
 * @codeCoverageIgnore
 */
class DiffProcessor
{
    /**
     * @var DirectoryReader
     */
    private $directoryReader;
    
    
    /**
     * DiffProcessorV3 constructor.
     *
     * @param DirectoryReader $directoryReader
     */
    public function __construct(DirectoryReader $directoryReader)
    {
        $this->directoryReader = $directoryReader;
    }
    
    
    /**
     * Collects all differences between the old and the new package.
     *
     * @param ComparePackage $newPackage
     * @param ComparePackage $oldPackage
     *
     * @return array
     */
    public function process(ComparePackage $newPackage, ComparePackage $oldPackage): array
    {
        $fileDifferences      = [];
        $directoryDifferences = [];
        $previousPackage      = null;
        
        $this->collectDifferences(
            $newPackage->root(),
            $oldPackage->root(),
            $fileDifferences,
            $directoryDifferences
        );
        
        $fileDifferences = empty($fileDifferences) ? [] : array_merge(...$fileDifferences);
        $differences     = array_merge($directoryDifferences, $fileDifferences);
        $differences     = array_unique($differences);
        
        sort($differences);
        
        return $differences;
    }
    
    
    /**
     * Iterates recursively through all old directories, compares them with the new package
     * and collects all differences.
     *
     * @param Directory $newPackageRoot
     * @param Directory $oldDirectory
     * @param array     $fileDifferences
     * @param array     $directoryDifferences
     */
    private function collectDifferences(
        Directory $newPackageRoot,
        Directory $oldDirectory,
        array &$fileDifferences,
        array &$directoryDifferences
    ): void {
        foreach ($oldDirectory->directories() as $directory) {
            if ($newDirectory = $newPackageRoot->findDirectory($directory)) {
                $this->collectDifferences($newPackageRoot, $directory, $fileDifferences, $directoryDifferences);
                
                $oldDirectoryFiles = $this->directoryReader->relativeFilePaths($directory, $directoryDifferences);
                $newDirectoryFiles = $this->directoryReader->relativeFilePaths($newDirectory, $directoryDifferences);
                
                $diff = array_diff(
                    $oldDirectoryFiles,
                    $newDirectoryFiles
                );
                if (!empty($diff)) {
                    $fileDifferences[] = array_values($diff);
                }
            } else {
                $directoryDifferences[] = "{$directory->relativePath()}/*";
            }
        }
    }
}