<?php
/* --------------------------------------------------------------
 PackageRepository.php 2020-04-21
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

namespace Gambio\Dragger\PackageComparator\Repositories;

use DirectoryIterator as Dir;
use Gambio\Dragger\PackageComparator\Exceptions\PathNotFoundException;
use Gambio\Dragger\PackageComparator\Factories\PackageFactory;
use Gambio\Dragger\PackageComparator\Model\ComparePackage;
use Gambio\Dragger\PackageComparator\Model\Directory;
use IteratorIterator as Iterator;
use function in_array;
use function str_replace;

/**
 * Class PackageRepository
 * @package Gambio\Dragger\PackageComparator\Repositories
 */
class PackageRepository
{
    /**
     * @var PackageFactory
     */
    private $factory;
    
    
    /**
     * PackageRepository constructor.
     *
     * @param PackageFactory $factory
     */
    public function __construct(PackageFactory $factory)
    {
        $this->factory = $factory;
    }
    
    
    /**
     * @param string $packageName
     * @param string $path
     * @param array  $excludes
     *
     * @return ComparePackage
     * @throws PathNotFoundException
     */
    public function getComparePackage(string $packageName, string $path, array $excludes = []): ComparePackage
    {
        if (!is_dir($path)) {
            $msg = "The given \$path ({$path}) was not found.";
            throw new PathNotFoundException($msg);
        }
        $realPath  = realpath($path);
        $path      = str_replace('\\', '/', $realPath ?? $path);
        $pathArray = explode('/', $path);
        $name      = array_pop($pathArray);
        
        $root = $this->directory($name, $path, $path, $excludes);
        
        return $this->factory->createComparePackage($packageName, $root);
    }
    
    
    /**
     * @param string $name
     * @param string $directoryPath
     * @param string $root
     * @param array  $excludes
     *
     * @return Directory
     */
    private function directory(string $name, string $directoryPath, string $root, array $excludes): Directory
    {
        $directories = $this->factory->createDirectories();
        $files       = $this->factory->createFiles();
        
        foreach ($this->iterator($directoryPath) as $dirEntry) {
            $relative = str_replace($root, '.', $dirEntry->getPathname());
            if ($dirEntry->isDot() || $this->isExcluded($relative, $excludes)) {
                continue;
            }
            if ($dirEntry->isDir()) {
                $directories->add(
                    $this->directory($dirEntry->getFilename(), $dirEntry->getPathname(), $root, $excludes)
                );
            }
            if ($dirEntry->isFile()) {
                $files->add(
                    $this->factory->createFile($dirEntry->getFilename(), $dirEntry->getPathname(), $relative)
                );
            }
        }
        $relativePath = str_replace($root, '.', $directoryPath);
    
        return $this->factory->createDirectory($name, $directoryPath, $relativePath, $directories, $files);
    }
    
    
    /**
     * @param string $relativePath
     * @param array  $excludes
     *
     * @return bool
     */
    private function isExcluded(string $relativePath, array $excludes): bool
    {
        return in_array($relativePath, $excludes, true);
    }
    
    
    /**
     * @param string $path
     *
     * @return Iterator|Dir[]
     */
    private function iterator(string $path): Iterator
    {
        return new Iterator(new Dir($path));
    }
}