<?php
/* --------------------------------------------------------------
 DirectoryReader.php 2020-04-27
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

namespace Gambio\Dragger\PackageComparator\Utility;

use Gambio\Dragger\PackageComparator\Model\Collections\Directories;
use Gambio\Dragger\PackageComparator\Model\Collections\Files;
use Gambio\Dragger\PackageComparator\Model\Directory;

/**
 * Class DirectoryReader
 * @package Gambio\Dragger\PackageComparator\Repositories
 *
 * @codeCoverageIgnore
 */
class DirectoryReader
{
    /**
     * Collects the relative file paths of all files in the given directory.
     *
     * @param Directory $directory
     * @param array     $excludes
     *
     * @return array
     */
    public function relativeFilePaths(Directory $directory, array $excludes = []): array
    {
        $files    = [];
        $excludes = array_map(
            static function (string $element): string {
                return rtrim($element, '/*');
            },
            $excludes
        );
        
        $this->readDirectories($directory->directories(), $files, $excludes);
        $this->collectFiles($directory->files(), $files, $excludes);
        
        return $files;
    }
    
    
    /**
     * Iterates recursively through all directories and collects the relative path of all files.
     *
     * @param Directories $directories
     * @param array       $collectedFiles
     * @param array       $excludes
     */
    private function readDirectories(Directories $directories, array &$collectedFiles, array $excludes): void
    {
        foreach ($directories as $directory) {
            if ($this->isExcluded($directory->relativePath(), $excludes)) {
                continue;
            }
            
            $this->readDirectories($directory->directories(), $collectedFiles, $excludes);
            $this->collectFiles($directory->files(), $collectedFiles, $excludes);
        }
    }
    
    
    /**
     * Collects the relative file paths.
     *
     * @param Files $files
     * @param array $collectedFiles
     * @param array $excludes
     */
    private function collectFiles(Files $files, array &$collectedFiles, array $excludes): void
    {
        foreach ($files as $file) {
            if (!$this->isExcluded($file->relativePath(), $excludes)) {
                $collectedFiles[] = $file->relativePath();
            }
        }
    }
    
    
    /**
     * Checks if given path is excluded.
     *
     * @param string $relativePath
     * @param array  $excludes
     *
     * @return bool
     */
    private function isExcluded(string $relativePath, array $excludes): bool
    {
        foreach ($excludes as $exclude) {
            if (strpos($relativePath, $exclude) === 0) {
                return true;
            }
        }
        
        return false;
    }
}