<?php
/* --------------------------------------------------------------
 CreateAccount.php 2019-06-18
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\Tools\Domain\GxInstaller\Steps;

use Gambio\Dragger\Tools\Domain\Exception\GxInstaller\InstallationFailedException;
use Gambio\Dragger\Tools\Domain\GxInstaller\Steps\CreateAccount\AddressBookQuery;
use Gambio\Dragger\Tools\Domain\GxInstaller\Steps\CreateAccount\AdminAccessQuery;
use Gambio\Dragger\Tools\Domain\GxInstaller\Steps\CreateAccount\ConfigurationsQuery;
use Gambio\Dragger\Tools\Domain\GxInstaller\Steps\CreateAccount\CreateAccountData;
use Gambio\Dragger\Tools\Domain\GxInstaller\Steps\CreateAccount\CustomersInfoQuery;
use Gambio\Dragger\Tools\Domain\GxInstaller\Steps\CreateAccount\CustomersQuery;
use PDO;
use PDOException;

/**
 * Class CreateAccount
 * @package Gambio\Dragger\Tools\Domain\GxInstaller\Steps
 */
class CreateAccount
{
    /**
     * @var PDO
     */
    private $pdo;
    
    
    /**
     * CreateAccount constructor.
     *
     * @param PDO $pdo
     */
    public function __construct(PDO $pdo)
    {
        $this->pdo = $pdo;
    }
    
    
    /**
     * Executes all step tasks.
     *
     * @param CreateAccountData $data
     *
     * @throws InstallationFailedException
     */
    public function run(CreateAccountData $data) : void
    {
        try {
            $this->pdo->beginTransaction();
            
            $this->customers($data);
            $this->customersInfo();
            $this->addressBook($data);
            $this->adminAccess();
            $this->configurations($data);
            
            $this->pdo->commit();
        } catch (PDOException $e) {
            $this->pdo->rollBack();
            
            throw new InstallationFailedException('CreateAccount failed', 0, $e);
        }
    }
    
    
    /**
     * Adds customer data.
     *
     * @param CreateAccountData $data
     */
    private function customers(CreateAccountData $data) : void
    {
        $query  = CustomersQuery::query();
        $params = CustomersQuery::params($data);
        
        $this->executePrepared($query, $params);
    }
    
    
    /**
     * Adds customer info data.
     */
    private function customersInfo() : void
    {
        $this->execute(CustomersInfoQuery::query());
    }
    
    
    /**
     * Adds address book data.
     *
     * @param CreateAccountData $data
     */
    private function addressBook(CreateAccountData $data) : void
    {
        $query  = AddressBookQuery::query();
        $params = AddressBookQuery::params($data);
        
        $this->executePrepared($query, $params);
    }
    
    
    /**
     * Adds admin access data.
     */
    private function adminAccess() : void
    {
        $this->execute(AdminAccessQuery::query());
    }
    
    
    /**
     * Adjusts create account data in configurations table.
     *
     * @param CreateAccountData $data
     */
    private function configurations(CreateAccountData $data) : void
    {
        $query  = ConfigurationsQuery::emailQuery();
        $params = ConfigurationsQuery::emailParams($data);
        $this->executePrepared($query, $params);
        
        $query  = ConfigurationsQuery::companyQuery();
        $params = ConfigurationsQuery::companyParams($data);
        $this->executePrepared($query, $params);
        
        $query  = ConfigurationsQuery::customerQuery();
        $params = ConfigurationsQuery::customerParams($data);
        $this->executePrepared($query, $params);
        
        $this->execute(ConfigurationsQuery::countryQuery());
        
        $query  = ConfigurationsQuery::finalizeQuery();
        $params = ConfigurationsQuery::finalizeParams($data);
        $this->executePrepared($query, $params);
    }
    
    
    /**
     * Executes the query.
     *
     * @param string $query
     *
     * @throws PDOException
     */
    private function execute(string $query) : void
    {
        $result = $this->pdo->exec($query);
        
        if ($result === false) {
            $this->fail($query);
        }
    }
    
    
    /**
     * Executes a prepared statement.
     *
     * @param string $query
     * @param array  $params
     *
     * @throws PDOException
     */
    private function executePrepared(string $query, array $params) : void
    {
        $statement = $this->pdo->prepare($query);
        $result    = $statement->execute($params);
        
        if (!$result) {
            $this->fail($query);
        }
    }
    
    
    /**
     * Triggers failure on invalid sql query.
     *
     * @param string $query
     *
     * @throws PDOException
     */
    private function fail(string $query) : void
    {
        $error = 'Could not execute query: ' . $query;
        throw new PDOException($error);
    }
}