<?php
/* --------------------------------------------------------------
 FixtureImporter.php 2020-04-15
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

namespace Gambio\Dragger\Tools\TestingToolbox;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DBALException;

/**
 * Class FixtureImporter
 * @package Gambio\Dragger\Tools\TestingToolbox
 */
class FixtureImporter
{
    /**
     * @var Connection
     */
    private $connection;
    
    
    /**
     * FixtureImporter constructor.
     *
     * @param Connection $connection
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }
    
    
    /**
     * Imports a database fixture.
     *
     * This method using the file path of the fixture file to import the data.
     * The returned array contains error information and is empty if everything succeed.
     *
     * @param string $fixturePath
     *
     * @return array
     */
    public function importFixture(string $fixturePath): array
    {
        if (!file_exists($fixturePath)) {
            return [
                [
                    'msg' => "Path of \$fixturePath ({$fixturePath}) not exists."
                ]
            ];
        }
        
        /** @noinspection PhpIncludeInspection */
        $fixture = include $fixturePath;
        if (!is_array($fixture)) {
            return [
                [
                    'msg' => 'Content of $fixturePath is not returning an array with fixture data.'
                ]
            ];
        }
        
        return $this->importFixtureData($fixture);
    }
    
    
    /**
     * Import a database fixture.
     *
     * This method using the raw fixture data to be imported.
     * The returned array contains error information and is empty if everything succeed.
     *
     * @param array $fixture
     *
     * @return array
     */
    public function importFixtureData(array $fixture): array
    {
        $errors = [];
        
        $this->disableForeignKeyCheck();
        foreach ($fixture as $table => $data) {
            $this->truncate($table);
            
            foreach ($data as $dataSet) {
                $keys = array_keys($dataSet);
                $keys = array_map([$this->connection, 'quoteIdentifier'], $keys);
                
                $dataSet = array_combine($keys, $dataSet);
                
                try {
                    $this->connection->insert($this->connection->quoteIdentifier($table), $dataSet);
                } catch (DBALException $e) {
                    $errors[] = [
                        'table' => $table,
                        'data'  => $dataSet,
                        'msg'   => $e->getMessage()
                    ];
                }
            }
        }
        $this->enableForeignKeyCheck();
        
        return $errors;
    }
    
    
    /**
     * Truncates the database table with the given name.
     *
     * @param string $table
     */
    private function truncate(string $table): void
    {
        $query = "TRUNCATE TABLE {$this->connection->quoteIdentifier($table)};";
        $this->execSilentFail($query);
    }
    
    
    /**
     * Enables the foreign key check.
     */
    private function enableForeignKeyCheck(): void
    {
        $query = 'SET FOREIGN_KEY_CHECKS=1;';
        $this->execSilentFail($query);
    }
    
    
    /**
     * Disables the foreign key check.
     */
    private function disableForeignKeyCheck(): void
    {
        $query = 'SET FOREIGN_KEY_CHECKS=0;';
        $this->execSilentFail($query);
    }
    
    
    /**
     * Executes the given sql and silently fail on error.
     *
     * @param string $query
     */
    private function execSilentFail(string $query): void
    {
        try {
            $this->connection->exec($query);
        } catch (DBALException $e) {
        }
    }
}