<?php /** @noinspection PhpIncompatibleReturnTypeInspection */
/* --------------------------------------------------------------
 PropertyFailureTest.php 2019-07-12
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Failure;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Property;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\Schema;
use PHPUnit\Framework\TestCase;
use function implode;

class PropertyFailureTest extends TestCase
{
    public const EXPECTED_MISSING_REQUIRED_SINGLE_MSG = <<<MSG
Node my -> prop:
Failed asserting that required property "bar" exists.
MSG;
    
    public const EXPECTED_MISSING_REQUIRED_MULTIPLE_MSG = <<<MSG
Node my -> prop:
Failed asserting that required properties "foo", "bar" exists.
MSG;
    
    public const EXPECTED_MISSING_REQUIRED_SCHEMA = <<<MSG
Node root (GXProduct):
Failed asserting that required properties "foo", "bar" exists.
MSG;
    
    
    public const EXPECTED_WRONG_TYPE_MSG = <<<MSG
Node my -> prop:
Failed asserting that 0.123 (double) is of type "string"
MSG;
    
    public const EXPECTED_NOT_LIST_BUT_MAP_MSG = <<<MSG
Node my -> prop:
Failed asserting that \$value is list. Got map.
MSG;
    
    public const EXPECTED_NOT_LIST_BUT_ELSE_MSG = <<<MSG
Node my -> prop:
Failed asserting that \$value is list. Got something else.
MSG;
    
    public const EXPECTED_NOT_MAP_BUT_LIST_MSG = <<<MSG
Node my -> prop:
Failed asserting that \$value is map. Got list.
MSG;
    
    public const EXPECTED_NOT_MAP_BUT_ELSE_MSG = <<<MSG
Node my -> prop:
Failed asserting that \$value is map. Got something else.
MSG;
    
    
    public function testForMissingRequiredProperties() : void
    {
        $prop            = $this->createObjectProperty('my -> prop', ['foo', 'bar']);
        $responseItemOne = [
            'foo' => 1,
            'baz' => 'string'
        ];
        $responseItemTwo = [
            'hello' => 'world'
        ];
        $failureOne      = PropertyFailure::missingRequired($prop, $responseItemOne);
        $failureTwo      = PropertyFailure::missingRequired($prop, $responseItemTwo);
        
        $errors = [];
        if (self::EXPECTED_MISSING_REQUIRED_SINGLE_MSG !== $failureOne->toString()) {
            $msg      = "Failed asserting that single missing required property returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_MISSING_REQUIRED_SINGLE_MSG . "\n\t";
            $msg      .= "Actual:\n{$failureOne->toString()}\n";
            $errors[] = $msg;
        }
        if (self::EXPECTED_MISSING_REQUIRED_MULTIPLE_MSG !== $failureTwo->toString()) {
            $msg      = "Failed asserting that multiple missing required property returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_MISSING_REQUIRED_MULTIPLE_MSG . "\n\t";
            $msg      .= "Actual:\n{$failureTwo->toString()}\n";
            $errors[] = $msg;
        }
        
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    public function testForMissingRequiredInSchema() : void
    {
        $schema       = $this->createSchema('GXProduct', ['foo', 'bar']);
        $responseItem = [
            'hello' => 'world'
        ];
        $failure      = PropertyFailure::missingRequired($schema, $responseItem);
        
        $this->assertEquals(self::EXPECTED_MISSING_REQUIRED_SCHEMA, $failure->toString());
    }
    
    
    public function testForWrongPropertyType() : void
    {
        $raw          = 0.123;
        $expectedType = 'string';
        $property     = $this->createScalarProperty('my -> prop', $expectedType);
        
        $failure = PropertyFailure::wrongType($property, $raw);
        
        $this->assertEquals(self::EXPECTED_WRONG_TYPE_MSG, $failure->toString());
    }
    
    
    public function testForNotList() : void
    {
        $property = $this->createProperty('my -> prop');
        
        $errors      = [];
        $mapFailure  = PropertyFailure::notList($property, true);
        $elseFailure = PropertyFailure::notList($property, false);
        
        if ($mapFailure->toString() !== self::EXPECTED_NOT_LIST_BUT_MAP_MSG) {
            $msg      = "Failed asserting that not list but map failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_LIST_BUT_MAP_MSG . "\n\t";
            $msg      .= "Actual:\n{$mapFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        if ($elseFailure->toString() !== self::EXPECTED_NOT_LIST_BUT_ELSE_MSG) {
            $msg      = "Failed asserting that not list but else failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_LIST_BUT_ELSE_MSG . "\n\t";
            $msg      .= "Actual:\n{$elseFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    public function testForNotMap() : void
    {
        $property = $this->createProperty('my -> prop');
        
        $errors      = [];
        $mapFailure  = PropertyFailure::notMap($property, true);
        $elseFailure = PropertyFailure::notMap($property, false);
        
        if ($mapFailure->toString() !== self::EXPECTED_NOT_MAP_BUT_LIST_MSG) {
            $msg      = "Failed asserting that not map but list failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_MAP_BUT_LIST_MSG . "\n\t";
            $msg      .= "Actual:\n{$mapFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        if ($elseFailure->toString() !== self::EXPECTED_NOT_MAP_BUT_ELSE_MSG) {
            $msg      = "Failed asserting that not map but else failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_MAP_BUT_ELSE_MSG . "\n\t";
            $msg      .= "Actual:\n{$elseFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    private function createProperty(string $nodePath) : Property
    {
        return $this->createConfiguredMock(
            Property::class, [
                               'nodePath' => $nodePath
                           ]
        );
    }
    
    
    private function createScalarProperty(string $nodePath, string $type) : ScalarProperty
    {
        return $this->createConfiguredMock(
            ScalarProperty::class, [
                                     'nodePath' => $nodePath,
                                     'type'     => $type,
                                 ]
        );
    }
    
    
    private function createObjectProperty(string $nodePath, array $required = []) : ObjectProperty
    {
        return $this->createConfiguredMock(
            ObjectProperty::class, [
                                     'nodePath' => $nodePath,
                                     'required' => $required
                                 ]
        );
    }
    
    
    private function createSchema(string $title, array $required) : Schema
    {
        return $this->createConfiguredMock(
            Schema::class, [
                             'name'     => $title,
                             'required' => $required
                         ]
        );
    }
}
