<?php /** @noinspection PhpIncompatibleReturnTypeInspection */
/* --------------------------------------------------------------
 SchemaFailureTest.php 2019-07-15
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Failure;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Property;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaListScalar;
use PHPUnit\Framework\TestCase;

class SchemaFailureTest extends TestCase
{
    public const EXPECTED_NOT_LIST_BUT_MAP_MSG = <<<MSG
Node root:
Failed asserting that response is list. Got map.
MSG;
    
    public const EXPECTED_NOT_LIST_BUT_ELSE_MSG = <<<MSG
Node root:
Failed asserting that response is list. Got something else.
MSG;
    
    public const EXPECTED_NOT_MAP_BUT_LIST_MSG = <<<MSG
Node root:
Failed asserting that response is map. Got list.
MSG;
    
    public const EXPECTED_NOT_MAP_BUT_ELSE_MSG = <<<MSG
Node root:
Failed asserting that response is map. Got something else.
MSG;
    
    public const EXPECTED_MISSING_IN_ROOT_SCHEMA_MSG = <<<MSG
Node root:
Failed asserting that property "propName" is defined in schema.
MSG;
    
    public const EXPECTED_MISSING_IN_CHILD_SCHEMA_MSG = <<<MSG
Node my -> prop:
Failed asserting that property "propName" is defined in schema.
MSG;
    
    
    public function testForNotList() : void
    {
        $errors      = [];
        $mapFailure  = SchemaFailure::notList(true);
        $elseFailure = SchemaFailure::notList(false);
        
        if ($mapFailure->toString() !== self::EXPECTED_NOT_LIST_BUT_MAP_MSG) {
            $msg      = "Failed asserting that not list but map failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_LIST_BUT_MAP_MSG . "\n\t";
            $msg      .= "Actual:\n{$mapFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        if ($elseFailure->toString() !== self::EXPECTED_NOT_LIST_BUT_ELSE_MSG) {
            $msg      = "Failed asserting that not list but else failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_LIST_BUT_ELSE_MSG . "\n\t";
            $msg      .= "Actual:\n{$elseFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    public function testForNotMap() : void
    {
        $errors      = [];
        $mapFailure  = SchemaFailure::notMap(true);
        $elseFailure = SchemaFailure::notMap(false);
        
        if ($mapFailure->toString() !== self::EXPECTED_NOT_MAP_BUT_LIST_MSG) {
            $msg      = "Failed asserting that not map but list failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_MAP_BUT_LIST_MSG . "\n\t";
            $msg      .= "Actual:\n{$mapFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        if ($elseFailure->toString() !== self::EXPECTED_NOT_MAP_BUT_ELSE_MSG) {
            $msg      = "Failed asserting that not map but else failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_NOT_MAP_BUT_ELSE_MSG . "\n\t";
            $msg      .= "Actual:\n{$elseFailure->toString()}\n";
            $errors[] = $msg;
        }
        
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    public function testForMissingInSchema() : void
    {
        $errors       = [];
        $rootFailure  = SchemaFailure::missingInSchema('propName');
        $childFailure = SchemaFailure::missingInSchema('propName', $this->createProperty('my -> prop'));
        
        if ($rootFailure->toString() !== self::EXPECTED_MISSING_IN_ROOT_SCHEMA_MSG) {
            $msg      = "Failed asserting that not missing in schema failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_MISSING_IN_ROOT_SCHEMA_MSG . "\n\t";
            $msg      .= "Actual:\n{$rootFailure->toString()}\n";
            $errors[] = $msg;
        }
        if ($childFailure->toString() !== self::EXPECTED_MISSING_IN_CHILD_SCHEMA_MSG) {
            $msg      = "Failed asserting that not missing in schema failure returns expected string.\n\t";
            $msg      .= "Expected:\n" . self::EXPECTED_MISSING_IN_CHILD_SCHEMA_MSG . "\n\t";
            $msg      .= "Actual:\n{$childFailure->toString()}\n";
            $errors[] = $msg;
        }
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    public function testForWrongTypeInSchemaListScalar() : void
    {
        $expected = <<<EXPECTED
Node: my-schema:
Failed asserting that hello (string) is of type "bool".
EXPECTED;
        
        $this->assertEquals(
            $expected, SchemaFailure::wrongType(SchemaListScalar::create('my-schema', 'bool'), 'hello')->toString()
        );
    }
    
    
    private function createProperty(string $nodePath) : Property
    {
        return $this->createConfiguredMock(
            Property::class, [
                               'nodePath' => $nodePath
                           ]
        );
    }
}
