<?php
/* --------------------------------------------------------------
 PropertyFactoryTest.php 2019-08-06
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory;

use Faker\Factory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Resolver\ReferenceParser;
use InvalidArgumentException;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

class PropertyFactoryTest extends TestCase
{
    public const SCALAR_DATA          = ['type' => 'string'];
    public const LIST_DATA            = [
        'type'  => 'array',
        'items' => [
            'type' => 'string'
        ]
    ];
    public const REFERENCED_LIST_DATA = [
        'type'  => 'array',
        'items' => [
            '$ref' => '#/definitions/categoryGroupPermissions'
        ]
    ];
    public const REFERENCED_DATA      = ['$ref' => '#/definitions/categoryGroupPermissions'];
    public const OBJECT_DATA          = [
        'type'       => 'object',
        'properties' => [
            'name' => [
                'type' => 'string',
            ],
            'foo'  => [
                'type'       => 'object',
                'properties' => [
                    'title' => [
                        'type' => 'string'
                    ]
                ]
            ]
        ]
    ];
    public const OBJECT_LIST_DATA     = [
        'type'  => 'array',
        'items' => [
            'type'       => 'object',
            'properties' => [
                'name' => [
                    'type' => 'string',
                ],
                'foo'  => [
                    'type'       => 'object',
                    'properties' => [
                        'title' => [
                            'type' => 'string'
                        ]
                    ]
                ]
            ]
        ]
    ];
    
    /**
     * @var PropertyFactory
     */
    private $factory;
    
    /**
     * @var MockObject|ReferenceParser
     */
    private $refParser;
    
    
    protected function setUp()
    {
        $this->refParser = $this->createMock(ReferenceParser::class);
        $this->refParser->method('parse')->willReturn(self::OBJECT_DATA);
        
        $this->factory = new PropertyFactory($this->refParser);
    }
    
    
    /**
     * @param string $fqn
     * @param array  $data
     *
     * @dataProvider propertiesDataProvider
     */
    public function testCreatesProperties(string $fqn, array $data) : void
    {
        $name = Factory::create('de_DE')->word;
        $this->assertInstanceOf($fqn, $this->factory->create($name, $data));
    }
    
    
    public function testReferencesTriggersRelatedParser() : void
    {
        $faker = Factory::create('de_DE');
        
        $this->refParser->expects($this->exactly(2))->method('parse');
        
        $this->factory->create('foo', self::REFERENCED_LIST_DATA);
        $this->factory->create('bar', self::REFERENCED_DATA);
    }
    
    
    public function testRejectsInvalidListPropData() : void
    {
        $this->expectException(InvalidArgumentException::class);
        $this->factory->create(
            'foo', [
                     'type'  => 'array',
                     'items' => [
                         'foo' => 'bar'
                     ]
                 ]
        );
    }
    
    
    public function propertiesDataProvider() : array
    {
        return [
            [ScalarProperty::class, self::SCALAR_DATA],
            [ListProperty::class, self::LIST_DATA],
            [ObjectListProperty::class, self::REFERENCED_LIST_DATA],
            [ObjectProperty::class, self::REFERENCED_DATA],
            [ObjectProperty::class, self::OBJECT_DATA],
            [ObjectListProperty::class, self::OBJECT_LIST_DATA],
        ];
    }
}
