<?php
/* --------------------------------------------------------------
 EndpointMapperTest.php 2019-07-03
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Repository;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Parameters;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Responses;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Parameter;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\SchemaReference;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\ParameterFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory\SimpleFactory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Response;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use UnexpectedValueException;

class EndpointMapperTest extends TestCase
{
    public const ENDPOINT_DATA = [
        '/categories/{category_id}' => [
            'get' => [
                'tags'        => [
                    'categories',
                ],
                'summary'     => 'Get Categories (Single)',
                'description' => 'Get a single category record through a GET request. This method supports all the GET parameters that are mentioned in the "Introduction" section of this documentation. To see an example usage take a look at `docs/REST/samples/category-service/fetch_category.php`',
                'operationId' => 'getCategory',
                'produces'    => [
                    'application/json; charset=utf-8',
                ],
                'parameters'  => [
                    [
                        'name'        => 'category_id',
                        'in'          => 'path',
                        'description' => 'The category ID',
                        'required'    => true,
                        'type'        => 'integer',
                        'format'      => 'int64',
                    ],
                ],
                'responses'   => [
                    200 => [
                        'description' => 'Upon success, returns a category',
                        'schema'      => [
                            '$ref' => '#/definitions/GXCategory',
                        ],
                    ],
                    404 => [
                        'description' => '(Category does not exist)',
                        'schema'      => [
                            '$ref' => '#/definitions/defaultErrorResponse',
                        ],
                    ],
                ],
            ],
        ],
    ];
    
    
    /**
     * @var EndpointMapper
     */
    private $mapper;
    
    /**
     * @var MockObject|SimpleFactory
     */
    private $simpleFactory;
    
    /**
     * @var MockObject|ParameterFactory
     */
    private $parameterFactory;
    
    /**
     * @var MockObject|Parameter
     */
    private $parameter;
    
    /**
     * @var MockObject|Parameters
     */
    private $parameters;
    
    /**
     * @var MockObject|Responses
     */
    private $responses;
    
    /**
     * @var MockObject|SchemaReference
     */
    private $referenceOne;
    
    /**
     * @var MockObject|SchemaReference
     */
    private $referenceTwo;
    
    /**
     * @var MockObject|Response
     */
    private $responseOne;
    
    /**
     * @var MockObject|Response
     */
    private $responseTwo;
    
    
    protected function setUp()
    {
        $this->simpleFactory    = $this->createMock(SimpleFactory::class);
        $this->parameterFactory = $this->createMock(ParameterFactory::class);
        $this->mapper           = new EndpointMapper($this->simpleFactory, $this->parameterFactory);
        
        $this->parameter    = $this->createMock(Parameter::class);
        $this->parameters   = $this->createMock(Parameters::class);
        $this->responses    = $this->createMock(Responses::class);
        $this->referenceOne = $this->createMock(SchemaReference::class);
        $this->referenceTwo = $this->createMock(SchemaReference::class);
        $this->responseOne  = $this->createMock(Response::class);
        $this->responseTwo  = $this->createMock(Response::class);
        
        $this->parameterFactory->method('createParameters')->willReturn($this->parameters);
        $this->simpleFactory->method('createResponses')->willReturn($this->responses);
        $this->parameterFactory->method('create')->willReturn($this->parameter);
        $this->simpleFactory->method('createSchemaReference')->will(
            $this->onConsecutiveCalls($this->referenceOne, $this->referenceTwo)
        );
        $this->simpleFactory->method('createResponse')->will(
            $this->onConsecutiveCalls($this->responseOne, $this->responseTwo)
        );
    }
    
    
    public function testMessagesFactoryToCreateModels() : void
    {
        $createParameterArgs = self::ENDPOINT_DATA['/categories/{category_id}']['get']['parameters'][0];
        $createParameterArgs = [$this->equalTo($createParameterArgs)];
        
        $createSchemaArgs = [
            [$this->equalTo(['$ref' => '#/definitions/GXCategory'])],
            [$this->equalTo(['$ref' => '#/definitions/defaultErrorResponse'])],
        ];
        
        $createResponseArgs = [
            [$this->equalTo(200), $this->equalTo($this->referenceOne)],
            [$this->equalTo(404), $this->equalTo($this->referenceTwo)],
        ];
        
        $this->parameterFactory->expects($this->once())->method('create')->with(...$createParameterArgs);
        $this->simpleFactory->expects($this->exactly(2))->method('createSchemaReference')->withConsecutive(
            ...$createSchemaArgs
        );
        $this->simpleFactory->expects($this->exactly(2))->method('createResponse')->withConsecutive(
            ...$createResponseArgs
        );
        
        $this->parameterFactory->expects($this->once())->method('createParameters');
        $this->simpleFactory->expects($this->once())->method('createResponses');
        
        $this->simpleFactory->expects($this->once())->method('createEndpoint')->with(
            $this->equalTo('/categories/{category_id}'), $this->equalTo('get'), $this->equalTo($this->parameters),
            $this->equalTo($this->responses)
        );
        
        $this->mapper->map(self::ENDPOINT_DATA);
    }
    
    
    public function testMessagesParameterItemsToCollection() : void
    {
        $this->parameters->expects($this->once())->method('add')->with($this->equalTo($this->parameter));
        
        $this->mapper->map(self::ENDPOINT_DATA);
    }
    
    
    public function testMessagesResponseItemsToCollection() : void
    {
        $this->responses->expects($this->exactly(2))->method('add')->withConsecutive(
            [$this->equalTo($this->responseOne)], [$this->equalTo($this->responseTwo)]
        );
        
        $this->mapper->map(self::ENDPOINT_DATA);
    }
    
    
    public function testRejectsEmptyData() : void
    {
        $this->expectException(UnexpectedValueException::class);
        
        $this->mapper->map([]);
    }
}