<?php
/* --------------------------------------------------------------
 SchemaMapperTest.php 2019-07-04
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Repository;

use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Properties;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Contracts\Property;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Factory;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\Schema;
use InvalidArgumentException;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

class SchemaMapperTest extends TestCase
{
    public const SCHEMA_DATA = [
        'title'      => 'GXCategory',
        'type'       => 'object',
        'required'   => [
            'name',
        ],
        'properties' => [
            'id'        => [
                'type' => 'integer',
            ],
            'dateAdded' => [
                'type' => 'string',
            ],
            'name'      => [
                '$ref' => '#/definitions/GXMultiLangOption',
            ],
        ]
    ];
    
    
    public const SCHEMA_LIST_DATA = [
        'title' => 'GXCategoryListing',
        'type'  => 'array',
        'items' => [
            'title'      => 'GXCategory',
            'type'       => 'object',
            'required'   => [
                'name',
            ],
            'properties' => [
                'id'        => [
                    'type' => 'integer',
                ],
                'dateAdded' => [
                    'type' => 'string',
                ],
                'name'      => [
                    '$ref' => '#/definitions/GXMultiLangOption',
                ],
            ]
        ]
    ];
    
    public const SCHEMA_LIST_SCALAR_DATA = [
        'title' => 'productLinksGetResponse',
        'type'  => 'integer',
    ];
    
    /**
     * @var SchemaMapper
     */
    private $mapper;
    
    /**
     * @var MockObject|Factory\SchemaFactory
     */
    private $schemaFactory;
    
    /**
     * @var MockObject|Factory\PropertyFactory
     */
    private $propertyFactory;
    
    /**
     * @var MockObject|Properties
     */
    private $properties;
    
    /**
     * @var MockObject|Property
     */
    private $propertyOne;
    
    /**
     * @var MockObject|Property
     */
    private $propertyTwo;
    
    /**
     * @var MockObject|Property
     */
    private $propertyThree;
    
    
    protected function setUp()
    {
        $this->schemaFactory   = $this->createMock(Factory\SchemaFactory::class);
        $this->propertyFactory = $this->createMock(Factory\PropertyFactory::class);
        $this->mapper          = new SchemaMapper($this->schemaFactory, $this->propertyFactory);
        
        $this->properties    = $this->createMock(Properties::class);
        $this->propertyOne   = $this->createMock(Property::class);
        $this->propertyTwo   = $this->createMock(Property::class);
        $this->propertyThree = $this->createMock(Property::class);
        
        $this->propertyFactory->method('createProperties')->willReturn($this->properties);
        $this->propertyFactory->method('create')->will(
            $this->onConsecutiveCalls($this->propertyOne, $this->propertyTwo, $this->propertyThree)
        );
    }
    
    
    public function testMessagesFactoryToCreateModels() : void
    {
        $createPropertyArgs = [
            [
                $this->equalTo('id'),
                $this->equalTo(
                    [
                        'type' => 'integer',
                    ]
                )
            ],
            [
                $this->equalTo('dateAdded'),
                $this->equalTo(
                    [
                        'type' => 'string',
                    ]
                )
            ],
            [
                $this->equalTo('name'),
                $this->equalTo(
                    [
                        '$ref' => '#/definitions/GXMultiLangOption',
                    ]
                )
            ]
        ];
        
        $this->propertyFactory->expects($this->once())->method('createProperties');
        $this->propertyFactory->expects($this->exactly(3))->method('create')->withConsecutive(
            ...$createPropertyArgs
        );
        $this->schemaFactory->expects($this->once())->method('createSchema')->with(
            $this->equalTo('GXCategory'), $this->equalTo($this->properties), $this->equalTo(['name'])
        );
        
        $this->mapper->map(self::SCHEMA_DATA);
    }
    
    
    public function testMessagesPropertyItemsToCollection() : void
    {
        $addArgs = [
            [$this->equalTo($this->propertyOne)],
            [$this->equalTo($this->propertyTwo)],
            [$this->equalTo($this->propertyThree)],
        ];
        
        $this->properties->expects($this->exactly(3))->method('add')->withConsecutive(...$addArgs);
        
        $this->mapper->map(self::SCHEMA_DATA);
    }
    
    
    public function testRequiredCanBeEmpty() : void
    {
        $data = self::SCHEMA_DATA;
        unset($data['GXCategory']['required']);
        
        $this->schemaFactory->expects($this->once())->method('createSchema')->with(
            $this->equalTo('GXCategory'), $this->equalTo($this->properties)
        );
        
        $this->mapper->map($data);
    }
    
    
    public function testMapsToListSchemaIfNecessary() : void
    {
        $schema = $this->createMock(Schema::class);
        $this->schemaFactory->method('createSchema')->willReturn($schema);
        
        $this->schemaFactory->expects($this->once())->method('createSchemaList')->with(
            $this->equalTo('GXCategoryListing'), $this->equalTo($schema)
        );
        
        $this->mapper->map(self::SCHEMA_LIST_DATA);
    }
    
    
    public function testRejectsEmptyData() : void
    {
        $this->expectException(InvalidArgumentException::class);
        $this->mapper->map([]);
    }
    
    
    public function testMapsToScalarListSchema() : void
    {
        $this->propertyFactory->expects($this->never())->method('createProperties');
        
        $this->schemaFactory->expects($this->once())->method('createSchemaListScalar')->with(
            $this->equalTo('productLinksGetResponse'), $this->equalTo('integer')
        );
        
        $this->mapper->mapList(self::SCHEMA_LIST_SCALAR_DATA);
    }
}