<?php
/* --------------------------------------------------------------
 PropertiesValidatorTest.php 2019-08-16
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema;

use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Properties;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectListProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ObjectProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

class PropertiesValidatorTest extends TestCase
{
    /**
     * @var PropertiesValidator
     */
    private $validator;
    
    /**
     * @var MockObject|ScalarPropertyValidator
     */
    private $scalarValidator;
    
    /**
     * @var Failures
     */
    private $failures;
    
    /**
     * @var MockObject|RequiredValidator
     */
    private $requiredValidator;
    
    
    protected function setUp()
    {
        $this->scalarValidator   = $this->createMock(ScalarPropertyValidator::class);
        $this->requiredValidator = $this->createMock(RequiredValidator::class);
        $this->failures          = Failures::create();
        $this->validator         = new PropertiesValidator(
            $this->scalarValidator, $this->requiredValidator, $this->failures
        );
    }
    
    
    public function testValidatesAllScalarValuesOfResponseModelWithScalarValidator() : void
    {
        $responseItem = $this->__complexResponseItem();
        
        $objAProp = ScalarProperty::create('obj-a', 'integer');
        $objBProp = ScalarProperty::create('obj-b', 'string');
        $objCProp = ScalarProperty::create('obj-c', 'bool');
        
        $objListBuzProp    = ScalarProperty::create('buz', 'string');
        $objListProperties = Properties::create();
        $objListProperties->add($objListBuzProp);
        $objListProp = ObjectListProperty::create('obj-list', $objListProperties);
        
        $objProperties = Properties::create();
        $objProperties->add($objAProp);
        $objProperties->add($objBProp);
        $objProperties->add($objCProp);
        $objProperties->add($objListProp);
        
        $objectListIdProp     = ScalarProperty::create('id', 'integer');
        $objectListNameProp   = ScalarProperty::create('name', 'string');
        $objectListStatusProp = ScalarProperty::create('status', 'bool');
        $objectListProperties = Properties::create();
        $objectListProperties->add($objectListIdProp);
        $objectListProperties->add($objectListNameProp);
        $objectListProperties->add($objectListStatusProp);
        
        $idProp         = ScalarProperty::create('id', 'integer');
        $labelProp      = ScalarProperty::create('label', 'string');
        $activeProp     = ScalarProperty::create('active', 'bool');
        $objectProp     = ObjectProperty::create('object', $objProperties);
        $listProp       = ListProperty::create('list', 'number');
        $objectListProp = ObjectListProperty::create('object-list', $objectListProperties);
        $properties     = Properties::create();
        $properties->add($idProp);
        $properties->add($labelProp);
        $properties->add($activeProp);
        $properties->add($objectProp);
        $properties->add($listProp);
        $properties->add($objectListProp);
        
        $argsExpectation = [
            [$this->equalTo($idProp), $this->equalTo(1)],
            [$this->equalTo($labelProp), $this->equalTo('bar')],
            [$this->equalTo($activeProp), $this->equalTo(false)],
            [$this->equalTo($objAProp), $this->equalTo(1)],
            [$this->equalTo($objBProp), $this->equalTo('123')],
            [$this->equalTo($objCProp), $this->equalTo(true)],
            [$this->equalTo($objListBuzProp), $this->equalTo('baz')],
            [$this->equalTo($objListBuzProp), $this->equalTo('bam')],
            [$this->equalTo($listProp), $this->equalTo([1, 2, 3])],
            [$this->equalTo($objectListIdProp), $this->equalTo(1)],
            [$this->equalTo($objectListNameProp), $this->equalTo('foo')],
            [$this->equalTo($objectListStatusProp), $this->equalTo(true)],
            [$this->equalTo($objectListIdProp), $this->equalTo(2)],
            [$this->equalTo($objectListNameProp), $this->equalTo('bar')],
            [$this->equalTo($objectListStatusProp), $this->equalTo(false)],
        ];
        
        $this->scalarValidator->expects($this->exactly(15))->method('validate')->withConsecutive(...$argsExpectation);
        
        $this->validator->validate($properties, $responseItem);
    }
    
    
    public function testValidatesRequiredPropertiesWithRequiredValidator() : void
    {
        $responseItem = [
            'object'      => $objItem = [
                'foo' => 'bar'
            ],
            'object-list' => [
                $objListItemOne = ['bar' => 'baz'],
                $objListItemTwo = ['bar' => 'bam'],
            ]
        ];
        $requiredObj  = ['foo'];
        $requiredList = ['bar'];
        
        $fooProp = ScalarProperty::create('foo', 'string');
        $barProp = ScalarProperty::create('bar', 'string');
        
        $objProperties = Properties::create();
        $objProperties->add($fooProp);
        
        $objListProperties = Properties::create();
        $objListProperties->add($barProp);
        
        $objProp     = ObjectProperty::create('object', $objProperties, $requiredObj);
        $objListProp = ObjectListProperty::create('object-list', $objListProperties, $requiredList);
        
        $properties = Properties::create();
        $properties->add($objProp);
        $properties->add($objListProp);
        
        $arguments = [
            [$this->equalTo($objProp), $this->equalTo($objItem)],
            [$this->equalTo($objListProp), $this->equalTo($objListItemOne)],
            [$this->equalTo($objListProp), $this->equalTo($objListItemTwo)],
        ];
        
        $this->requiredValidator->expects($this->exactly(3))->method('validate')->withConsecutive(...$arguments);
        $this->validator->validate($properties, $responseItem);
    }
    
    
    public function testValidatesResponseItemTypeByList() : void
    {
        $this->failures->reset();
        $data   = $this->__wrongList();
        $errors = [];
        
        if ($this->validator->validate($data[0], $data[1])) {
            $errors[] = 'Failed asserting that validation with wrong list type fails.';
        }
        if (2 !== $this->failures->count()) {
            $errors[] = 'Failed asserting that validation with wrong list collects all errors.';
        }
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    private function __wrongList() : array
    {
        $objListProperties = Properties::create();
        $objListProperties->add(ScalarProperty::create('foo', 'string'));
        $objListProperties->add(ScalarProperty::create('baz', 'int'));
        
        $properties = Properties::create();
        $properties->add(ObjectListProperty::create('test', $objListProperties));
        $properties->add(ObjectListProperty::create('bar', $objListProperties));
        
        $responseItem = [
            'test' => [
                'foo' => 'bar',
                'baz' => 123
            ],
            'bar'  => [
                'foo' => 'bar',
                'baz' => 123
            ]
        ];
        
        return [$properties, $responseItem];
    }
    
    
    public function testValidatesResponseItemTypeByMap() : void
    {
        $this->failures->reset();
        $data   = $this->__wrongMap();
        $errors = [];
        
        if ($this->validator->validate($data[0], $data[1])) {
            $errors[] = 'Failed asserting that validation with wrong list type fails.';
        }
        if (1 !== $this->failures->count()) {
            $errors[] = 'Failed asserting that validation with wrong list collects all errors.';
        }
        $this->assertCount(0, $errors, implode(PHP_EOL, $errors));
    }
    
    
    private function __wrongMap() : array
    {
        $objProperties = Properties::create();
        $objProperties->add(ScalarProperty::create('foo', 'string'));
        $objProperties->add(ScalarProperty::create('baz', 'int'));
        
        $properties = Properties::create();
        $properties->add(ObjectProperty::create('test', $objProperties));
        
        $responseItem = [
            'test' => [
                [
                    'foo' => 'bar',
                    'baz' => 123
                ]
            ]
        ];
        
        return [$properties, $responseItem];
    }
    
    
    public function testValidatesResponseItemsThatAreNotDefined() : void
    {
        $responseItem = [
            'foo' => 'bar'
        ];
        
        $properties = Properties::create();
        $properties->add(ObjectProperty::create('test', Properties::create()));
        
        $this->validator->validate($properties, $responseItem);
        $this->assertEquals(1, $this->failures->count());
    }
    
    
    /**
     * @param Properties $properties
     * @param array      $responseItem
     *
     * @dataProvider failingValidationDataProvider
     */
    public function testValidationFailsOnError(Properties $properties, array $responseItem) : void
    {
        $this->assertFalse($this->validator->validate($properties, $responseItem));
    }
    
    
    public function failingValidationDataProvider() : array
    {
        return [
            'wrong map'  => $this->__wrongMap(),
            'wrong list' => $this->__wrongList()
        ];
    }
    
    
    public function testValidationFailsIfScalarValidatorFails() : void
    {
        $responseItem = $this->__complexResponseItem();
        $properties   = $this->__complexProperties();
        
        // with config above, the validator is used 15 times
        // so i will create an array with sample returns where at least one validation returns falwswe
        // so the final result expects to be false too.
        
        $scalarValidateReturns = [];
        $falseGenerated        = false;
        for ($i = 0; $i < 15; $i++) {
            $rng = \random_int(0, 1) === 0;
            if (!$rng) {
                $falseGenerated = true;
            }
            $scalarValidateReturns[] = $rng;
        }
        // if no record is false, set last record to false
        if (!$falseGenerated) {
            $scalarValidateReturns[count($scalarValidateReturns) - 1] = false;
        }
        $this->scalarValidator->method('validate')->will($this->onConsecutiveCalls(...$scalarValidateReturns));
        
        $this->assertFalse($this->validator->validate($properties, $responseItem));
    }
    
    
    public function testValidationsFailsIfRequiredValidatorFails() : void
    {
        $responseItem = [
            'object-list' => [
                $objListItemOne = ['bar' => 'baz'],
                $objListItemTwo = ['bar' => 'bam'],
            ]
        ];
        $requiredList = ['bar'];
        $barProp      = ScalarProperty::create('bar', 'string');
        
        $objListProperties = Properties::create();
        $objListProperties->add($barProp);
        
        $objListProp = ObjectListProperty::create('object-list', $objListProperties, $requiredList);
        
        $properties = Properties::create();
        $properties->add($objListProp);
        
        $this->requiredValidator->method('validate')->will($this->onConsecutiveCalls(true, false));
        $this->assertFalse($this->validator->validate($properties, $responseItem));
    }
    
    
    private function __complexResponseItem() : array
    {
        return [
            'id'          => 1,
            'label'       => 'bar',
            'active'      => false,
            'object'      => [
                'obj-a'    => 1,
                'obj-b'    => '123',
                'obj-c'    => true,
                'obj-list' => [
                    ['buz' => 'baz'],
                    ['buz' => 'bam'],
                ]
            ],
            'list'        => [1, 2, 3],
            'object-list' => [
                [
                    'id'     => 1,
                    'name'   => 'foo',
                    'status' => true
                ],
                [
                    'id'     => 2,
                    'name'   => 'bar',
                    'status' => false
                ]
            ]
        ];
    }
    
    
    private function __complexProperties()
    {
        // copied from ::testValidatesAllScalarValuesOfResponseModelWithScalarValidator
        
        $objAProp = ScalarProperty::create('obj-a', 'integer');
        $objBProp = ScalarProperty::create('obj-b', 'string');
        $objCProp = ScalarProperty::create('obj-c', 'bool');
        
        $objListBuzProp    = ScalarProperty::create('buz', 'string');
        $objListProperties = Properties::create();
        $objListProperties->add($objListBuzProp);
        $objListProp = ObjectListProperty::create('obj-list', $objListProperties);
        
        $objProperties = Properties::create();
        $objProperties->add($objAProp);
        $objProperties->add($objBProp);
        $objProperties->add($objCProp);
        $objProperties->add($objListProp);
        
        $objectListIdProp     = ScalarProperty::create('id', 'integer');
        $objectListNameProp   = ScalarProperty::create('name', 'string');
        $objectListStatusProp = ScalarProperty::create('status', 'bool');
        $objectListProperties = Properties::create();
        $objectListProperties->add($objectListIdProp);
        $objectListProperties->add($objectListNameProp);
        $objectListProperties->add($objectListStatusProp);
        
        $idProp         = ScalarProperty::create('id', 'integer');
        $labelProp      = ScalarProperty::create('label', 'string');
        $activeProp     = ScalarProperty::create('active', 'bool');
        $objectProp     = ObjectProperty::create('object', $objProperties);
        $listProp       = ListProperty::create('list', 'number');
        $objectListProp = ObjectListProperty::create('object-list', $objectListProperties);
        $properties     = Properties::create();
        $properties->add($idProp);
        $properties->add($labelProp);
        $properties->add($activeProp);
        $properties->add($objectProp);
        $properties->add($listProp);
        $properties->add($objectListProp);
        
        return $properties;
    }
}
