<?php
/* --------------------------------------------------------------
 SchemaValidatorTest.php 2019-08-16
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2019 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Dragger\ApiTesting\SwaggerLike\Validator\Schema;

use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\Failures;
use Gambio\Dragger\ApiTesting\SwaggerLike\Failure\SimpleFailure;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Collection\Properties;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Property\ScalarProperty;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\Schema;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaList;
use Gambio\Dragger\ApiTesting\SwaggerLike\Model\Schema\SchemaListScalar;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

class SchemaValidatorTest extends TestCase
{
    /**
     * @var SchemaValidator
     */
    private $validator;
    
    /**
     * @var MockObject|PropertiesValidator
     */
    private $propertiesValidator;
    
    /**
     * @var MockObject|RequiredValidator
     */
    private $requiredValidator;
    
    /**
     * @var Failures
     */
    private $failures;
    
    
    protected function setUp()
    {
        $this->propertiesValidator = $this->createMock(PropertiesValidator::class);
        $this->requiredValidator   = $this->createMock(RequiredValidator::class);
        $this->failures            = Failures::create();
        $this->validator           = new SchemaValidator(
            $this->propertiesValidator, $this->requiredValidator, $this->failures
        );
    }
    
    
    public function testValidatesRequiredSchemaFieldsWithRequiredValidator() : void
    {
        $response   = [
            'foo' => 'bar'
        ];
        $required   = ['foo'];
        $properties = Properties::create();
        $properties->add(ScalarProperty::create('foo', 'string'));
        $schema = Schema::create('schema', $properties, $required);
        
        $this->requiredValidator->expects($this->once())->method('validate')->with(
            $this->equalTo($schema), $this->equalTo($response)
        );
        
        $this->validator->validate($schema, $response);
    }
    
    
    public function testValidatesPropertiesWithPropertiesValidator() : void
    {
        $response   = [
            'foo' => 'bar'
        ];
        $required   = ['foo'];
        $properties = Properties::create();
        $properties->add(ScalarProperty::create('foo', 'string'));
        $schema = Schema::create('schema', $properties, $required);
        
        $this->propertiesValidator->expects($this->once())->method('validate')->with(
            $this->equalTo($properties), $this->equalTo($response)
        );
        
        $this->validator->validate($schema, $response);
    }
    
    
    public function testValidatesRequiredSchemaListFieldsWithRequiredValidator() : void
    {
        $response   = [
            ['foo' => 'bar'],
            ['foo' => 'baz'],
        ];
        $required   = ['foo'];
        $properties = Properties::create();
        $properties->add(ScalarProperty::create('foo', 'string'));
        $schema     = Schema::create('schema', $properties, $required);
        $schemaList = SchemaList::create('schema-list', $schema);
        
        $this->requiredValidator->expects($this->exactly(2))->method('validate')->withConsecutive(
            [$this->equalTo($schema), $this->equalTo(['foo' => 'bar'])],
            [$this->equalTo($schema), $this->equalTo(['foo' => 'baz'])]
        );
        
        $this->validator->validate($schemaList, $response);
    }
    
    
    public function testValidatesListPropertiesWithPropertiesValidator() : void
    {
        $response   = [
            ['foo' => 'bar'],
            ['foo' => 'baz'],
        ];
        $required   = ['foo'];
        $properties = Properties::create();
        $properties->add(ScalarProperty::create('foo', 'string'));
        $schema     = Schema::create('schema', $properties, $required);
        $schemaList = SchemaList::create('schema-list', $schema);
        
        $arguments = [
            [$this->equalTo($properties), ['foo' => 'bar']],
            [$this->equalTo($properties), ['foo' => 'baz']],
        ];
        $this->propertiesValidator->expects($this->exactly(2))->method('validate')->withConsecutive(...$arguments);
        
        $this->validator->validate($schemaList, $response);
    }
    
    
    public function testValidationFailsIfSchemaIsNotMap() : void
    {
        $response   = [
            ['foo' => 'bar'],
            ['foo' => 'baz'],
        ];
        $properties = Properties::create();
        $properties->add(ScalarProperty::create('foo', 'string'));
        $schema = Schema::create('schema', $properties);
        $this->validator->validate($schema, $response);
        
        $this->assertEquals(1, $this->failures->count());
    }
    
    
    public function testValidationFailsIfSchemaListIsNotList() : void
    {
        $response   = [
            'foo' => 'bar'
        ];
        $properties = Properties::create();
        $properties->add(ScalarProperty::create('foo', 'string'));
        $schema     = Schema::create('schema', $properties);
        $schemaList = SchemaList::create('schema-list', $schema);
        $this->validator->validate($schemaList, $response);
        
        $this->assertEquals(1, $this->failures->count());
    }
    
    
    /**
     * @param bool             $expected
     * @param SchemaListScalar $schema
     * @param array            $response
     *
     * @dataProvider scalarListSchemaDataProvider
     */
    public function testValidatesScalarListSchema(bool $expected, SchemaListScalar $schema, array $response) : void
    {
        $this->assertEquals($expected, $this->validator->validate($schema, $response));
    }
    
    
    public function testFailuresStringCanBeAccessed() : void
    {
        $message = 'foo';
        $this->failures->add(SimpleFailure::create($message));
        $this->assertEquals($message, $this->validator->failures());
    }
    
    
    public function testValidationResetsFailuresCollection() : void
    {
        $message = 'foo';
        $this->failures->add(SimpleFailure::create($message));
        $this->validator->validate($this->createMock(Schema::class), ['foo' => 'bar']);
        
        $this->assertEquals('', $this->validator->failures());
    }
    
    
    public function scalarListSchemaDataProvider() : array
    {
        $validString = [
            true,
            SchemaListScalar::create('foo', 'string'),
            ['foo', 'bar']
        ];
        $validNumber = [
            true,
            SchemaListScalar::create('foo', 'number'),
            [0, 1, 2.34]
        ];
        $invalid     = [
            false,
            SchemaListScalar::create('foo', 'number'),
            ['1', 2.34]
        ];
        $invalidList = [
            false,
            SchemaListScalar::create('foo', 'string'),
            ['foo' => 'bar']
        ];
        
        return [
            $validString,
            $validNumber,
            $invalid,
            $invalidList
        ];
    }
}
